@@ -24,9 +24,6 @@ int Read_Command_Line(option *io, int argc, char **argv)
   int i;
   int writemode;
 
-  PhyML_Printf("\n. Command line: ");
-  for(i=0;i<argc;i++) PhyML_Printf("%s ",argv[i]);
-  PhyML_Printf("\n");
   writemode = WRITE;
 
   if(argc == 1) Exit("\n. No argument was passed to the program. Please check the documentation. \n");
@@ -129,6 +126,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
       {"weights",             required_argument,NULL,79},
       {"tbe",                 no_argument,NULL,80},
       {"leave_duplicates",    no_argument,NULL,81},
+      {"precision",           required_argument,NULL,82},
       {0,0,0,0}
     };
 
@@ -146,6 +144,19 @@ int Read_Command_Line(option *io, int argc, char **argv)
 
       switch(c)
 	{
+	case 82 :
+	  {
+	    if ((!atoi(optarg)) || (atoi(optarg) < 1) || (atoi(optarg) > DECIMAL_DIG -3))
+	      {
+		    PhyML_Printf("\n. The number of digits must be [1 - %d]\n", DECIMAL_DIG -3);
+		    Exit("\n");
+	      }
+	    else 
+	      {
+		io->precision = atoi(optarg);
+	      }
+	    break;
+	  }
 	case 81 :
 	  {
 	    io->leave_duplicates = YES;
@@ -153,7 +164,9 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  }
 	case 80 :
 	  {
-	    io->tbe_bootstrap = YES;
+	    io->do_tbe = YES;
+	    io->do_boot = NO;
+            io->do_alrt = NO;
 	    break;
 	  }
         case 79:
@@ -250,6 +263,16 @@ int Read_Command_Line(option *io, int argc, char **argv)
             Free_Input(io);
             DATE_XML(optarg);
             return 0;
+
+#elif defined(PHYREX)
+
+            Free_Optimiz(io->mod->s_opt);
+            M4_Free_M4_Model(io->mod->m4mod);
+            Free_Model_Basic(io->mod);
+            Free_Input(io);
+            PHYREX_XML(optarg);
+            return 0;
+            
 #endif
             break;
 	  }
@@ -705,8 +728,8 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  }
 	case 18 :
 	  {
-	    io->mod->s_opt->steph_spr = 0;
-	    io->mod->s_opt->greedy    = 1;
+	    io->mod->s_opt->steph_spr = NO;
+	    io->mod->s_opt->greedy    = YES;
 	    break;
 	  }
 	case 19 :
@@ -716,12 +739,12 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  }
 	case 20 :
 	  {
-	    io->mod->s_opt->random_input_tree = 1;
+	    io->mod->s_opt->random_input_tree = YES;
 	    break;
 	  }
 	case 21 :
 	  {
-	    io->mod->s_opt->random_input_tree = 1;
+	    io->mod->s_opt->random_input_tree = YES;
 	    io->mod->s_opt->n_rand_starts = atoi(optarg);
 	    if(io->mod->s_opt->n_rand_starts < 1) Exit("\n. Number of random starting trees must be > 0.\n\n");
 	  }
@@ -815,174 +838,209 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	    break;
 	  }
 	case 'm': case 5 :
-	  {
-	    int i;
-	    
-	    For(i,strlen(optarg)) Uppercase(optarg+i);
-	    
-	    if(!isalpha(optarg[0]))
-	      {
-		strcpy(io->mod->custom_mod_string->s,optarg);
-		
-		if(strlen(io->mod->custom_mod_string->s) != 6)
-		  {
-		    Warn_And_Exit("\n. The string should be of length 6.\n");
-		  }
-		else
-		  {
-		    /* Make_Custom_Model(io->mod); */
-		    /* Translate_Custom_Mod_String(io->mod); */
-		  }
-		
-		io->datatype              = NT;
-		io->mod->whichmodel       = CUSTOM;
-		strcpy(io->mod->modelname->s, "custom");
-		io->mod->s_opt->opt_kappa = NO;
-		io->mod->s_opt->opt_rr    = YES;
-	      }
-	    
+	  {            
+	    if (!isalpha(optarg[0]))
+              {
+                if(strchr(optarg,',') == NULL)
+                  {
+                    strcpy(io->mod->custom_mod_string->s,optarg);
+                    if (strlen(io->mod->custom_mod_string->s) != 6)
+                      {
+                        Warn_And_Exit("\n. The custom model string should be of length 6.\n");
+                      }
+                    
+                    io->datatype              = NT;
+                    io->mod->whichmodel       = CUSTOM;
+                    strcpy(io->mod->modelname->s, "custom");
+                    io->mod->s_opt->opt_kappa = NO;
+                    io->mod->s_opt->opt_rr    = YES;
+                  }
+                else
+                  {
+                    phydbl v;
+                    int n_rr;
+                    const char *d = ",";
+                    char *tok = strtok(optarg, d);
+                    
+                    io->datatype           = NT;
+                    io->mod->ns            = 4;
+                    io->mod->whichmodel    = GTR;
+                    io->mod->s_opt->opt_rr = NO;                    
+
+                    io->mod->r_mat = (t_rmat *) Make_Rmat (io->mod->ns);
+                    Init_Rmat (io->mod->r_mat);
+                    Make_Custom_Model (io->mod);
+
+                    n_rr = 0;
+                    while(tok && n_rr < 6)
+                      {
+                        v = strtod(tok,NULL);
+                        if (v != 0)
+                          {
+                            io->mod->r_mat->rr->v[n_rr] = v;
+                            io->mod->r_mat->rr_val->v[n_rr] = log(v);
+                          }
+                        else
+                          {
+                            PhyML_Printf("\n. Invalid relative rate parameter value: '%s'.\n", tok);
+                            Exit("\n");
+                          }
+                        tok = strtok (NULL,d);
+                        n_rr++;
+                      }
+                    assert(n_rr == 5);
+                  }
+              }
 	    else if (strcmp(optarg, "JC69") == 0)
-	      {
-		io->datatype              = NT;
-		io->mod->whichmodel       = JC69;
-	      }
+              {
+                io->datatype              = NT;
+                io->mod->ns               = 4;
+                io->mod->whichmodel       = JC69;
+              }
 	    else if(strcmp(optarg, "K80") == 0)
-	      {
-		io->datatype              = NT;
-		io->mod->whichmodel       = K80;
-	      }
+              {
+                io->datatype              = NT;
+                io->mod->ns               = 4;
+                io->mod->whichmodel       = K80;
+              }
 	    else if(strcmp(optarg, "F81") == 0)
-	      {
-		io->datatype              = NT;
-		io->mod->whichmodel       = F81;
-	      }
+              {
+                io->datatype              = NT;
+                io->mod->ns               = 4;
+                io->mod->whichmodel       = F81;
+              }
 	    else if (strcmp(optarg, "HKY85") == 0)
-	      {
-		io->datatype              = NT;
-		io->mod->whichmodel       = HKY85;
-	      }
+              {
+                io->datatype              = NT;
+                io->mod->ns               = 4;
+                io->mod->whichmodel       = HKY85;
+              }
 	    else if(strcmp(optarg, "F84") == 0)
-	      {
-		io->datatype              = NT;
-		io->mod->whichmodel       = F84;
-	      }
+              {
+                io->datatype              = NT;
+                io->mod->ns               = 4;
+                io->mod->whichmodel       = F84;
+              }
 	    else if (strcmp (optarg,"TN93") == 0)
-	      {
-		io->datatype              = NT;
-		io->mod->whichmodel       = TN93;
-	      }
-	    else if(strcmp (optarg, "GTR") == 0)
-	      {
-		io->datatype              = NT;
-		io->mod->whichmodel       = GTR;
-	      }
+              {
+                io->datatype              = NT;
+                io->mod->ns               = 4;
+                io->mod->whichmodel       = TN93;
+              }
+	    else if(strncmp (optarg, "GTR", 3) == 0)
+              {
+                io->datatype              = NT;
+                io->mod->ns               = 4;
+                io->mod->whichmodel       = GTR;
+                io->mod->s_opt->opt_rr    = YES;                
+              }
 	    else if(strcmp(optarg, "DAYHOFF") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = DAYHOFF;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = DAYHOFF;
+              }
 	    else if(strcmp (optarg, "JTT") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = JTT;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = JTT;
+              }
 	    else if(strcmp(optarg, "MTREV") == 0)
-	      {
-		io->datatype             = AA;
-		io->mod->ns              = 20;
-		io->mod->whichmodel      = MTREV;
-	      }
+              {
+                io->datatype             = AA;
+                io->mod->ns              = 20;
+                io->mod->whichmodel      = MTREV;
+              }
 	    else if(strcmp (optarg, "LG") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = LG;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = LG;
+              }
 	    else if(strcmp (optarg, "WAG") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = WAG;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = WAG;
+              }
 	    else if(strcmp(optarg, "DCMUT") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = DCMUT;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = DCMUT;
+              }
 	    else if(strcmp (optarg, "RTREV") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = RTREV;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = RTREV;
+              }
 	    else if(strcmp(optarg, "CPREV") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = CPREV;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = CPREV;
+              }
 	    else if(strcmp(optarg, "VT") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = VT;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = VT;
+              }
 	    else if(strcmp(optarg, "BLOSUM62") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = BLOSUM62;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = BLOSUM62;
+              }
 	    else if(strcmp(optarg, "MTMAM") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = MTMAM;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = MTMAM;
+              }
 	    else if (strcmp(optarg,"MTART") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = MTART;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = MTART;
+              }
 	    else if (strcmp(optarg,"HIVW") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = HIVW;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = HIVW;
+              }
 	    else if(strcmp(optarg, "HIVB") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = HIVB;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = HIVB;
+              }
 	    else if(strcmp(optarg, "AB") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = AB;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = AB;
+              }
 	    else if (strcmp(optarg, "CUSTOM") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = CUSTOMAA;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = CUSTOMAA;
+              }
 	    else if(strcmp(optarg, "FLU") == 0)
-	      {
-		io->datatype              = AA;
-		io->mod->ns               = 20;
-		io->mod->whichmodel       = FLU;
-	      }
+              {
+                io->datatype              = AA;
+                io->mod->ns               = 20;
+                io->mod->whichmodel       = FLU;
+              }
 	    else
-	      {
-		PhyML_Printf("\n. The model name is incorrect. Please see the documentation.\n");
-		Exit("\n");
-	      }
+              {
+                PhyML_Printf("\n. The model name is incorrect. Please see the documentation.\n");
+                Exit("\n");
+              }
 
 	    Set_Model_Name(io->mod);
 	    
@@ -1009,7 +1067,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	    else
 	      {
 		io->mod->ras->alpha->v = (phydbl)atof(optarg);
-		io->mod->s_opt->opt_alpha  = 0;
+		io->mod->s_opt->opt_alpha  = NO;
 	      }
 	    break;
 	  }
@@ -1027,9 +1085,10 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	      {
 		if((int)String_To_Dbl(optarg) > 0)
 		  {
-		    io->ratio_test       = 0;
-		    io->mod->bootstrap   = (int)atoi(optarg);
-		    io->print_boot_trees = 1;
+                    io->do_alrt           = NO;
+		    io->ratio_test        = 0;
+		    io->n_boot_replicates = (int)atoi(optarg);
+		    io->print_boot_trees  = 1;
 		    
 		    if(io->n_data_sets > 1)
 		      {
@@ -1042,13 +1101,17 @@ int Read_Command_Line(option *io, int argc, char **argv)
 		  }
 		else if (atoi(optarg)==0)
 		  {
-		    io->mod->bootstrap = 0;
-		    io->ratio_test     = 0;
+		    io->do_alrt    = NO;
+                    io->do_tbe     = NO;
+                    io->do_boot    = NO;
+		    io->ratio_test = 0;
 		  }
 		else
 		  {
-		    io->mod->bootstrap = 0;
-		    io->ratio_test     = -(int)atoi(optarg);
+                    io->do_alrt = YES;
+                    io->do_tbe  = NO;
+                    io->do_boot = NO;
+		    io->ratio_test = -(int)atoi(optarg);
 		  }
 	      }
 	    break;
@@ -1215,37 +1278,63 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  
 	case 't':case 11:
 	  {
-	    if ((io->mod->whichmodel != JC69) &&
-		(io->mod->whichmodel != F81)  &&
-		(io->mod->whichmodel != GTR))
-	      {
-		if ((strcmp(optarg, "e") == 0) ||
-		    (strcmp(optarg, "E") == 0) ||
-		    (strcmp(optarg, "estimated") == 0) ||
-		    (strcmp(optarg, "ESTIMATED") == 0))
-		  {
-		    io->mod->kappa->v              = 4.0;
-		    io->mod->s_opt->opt_kappa      = YES;
-		    if (io->mod->whichmodel == TN93)
-		      io->mod->s_opt->opt_lambda   = YES;
-		  }
-		else
-		  {
-		    if (atof(optarg) < .0)
-		      {
-			char choix;
-			PhyML_Printf("\n. The ts/tv ratio must be a positive number\n");
-			PhyML_Printf("\n. Type any key to exit.\n");
-			if(!scanf("%c",&choix)) Exit("\n");
-			Exit("\n");
-		      }
-		    else
-		      {
-			io->mod->kappa->v = (phydbl)atof(optarg);
-			io->mod->s_opt->opt_kappa  = 0;
-			io->mod->s_opt->opt_lambda = 0;
-		      }
-		  }
+            if ((io->mod->whichmodel != JC69) && (io->mod->whichmodel != F81) && (io->mod->whichmodel != GTR))
+              {
+                if ((strcmp(optarg, "e") == 0) ||
+                    (strcmp(optarg, "E") == 0) ||
+                    (strcmp(optarg, "estimated") == 0) ||
+                    (strcmp(optarg, "ESTIMATED") == 0))
+                  {
+                    io->mod->kappa->v = 4.0;
+                    io->mod->s_opt->opt_kappa = YES;
+                    if (io->mod->whichmodel == TN93)
+                      io->mod->s_opt->opt_lambda   = YES;
+                  }
+                else
+                  {
+                    io->mod->s_opt->opt_kappa  = NO;
+                    io->mod->s_opt->opt_lambda = NO;
+                    
+                    // Added the 2 TsTv ratios for TN93
+                    // lambda is the ratio of both TsTv ratios
+                    // kappa is the mean of both TsTv ratios
+                    if (io->mod->whichmodel == TN93)
+                      {
+                        double TsTvPur, TsTvPyr;
+                        if(!isalpha(optarg[0]))
+                          {
+                            sscanf(optarg,"%lf,%lf",&TsTvPur,&TsTvPyr);
+                          }
+                        else
+                          {
+                            PhyML_Fprintf(stderr,"\n. The TN93 model requires two Ts/Tv ratios.\n");
+                            Exit("\n");
+                          }
+                        if ( (TsTvPur < .0) || (TsTvPyr < .0) )
+                          {
+                            PhyML_Fprintf(stderr,"\n. The Ts/Tv ratio must be a positive number.\n");
+                            Exit("\n");
+                          }
+                        io->mod->lambda->v = (phydbl)(TsTvPur / TsTvPyr);
+                        io->mod->kappa->v = (phydbl)((TsTvPur + TsTvPyr)/2.);
+                      }
+                    else
+                      {
+                        // -- End TN93 rates for purines & pyrimidines
+                        if (atof(optarg) < .0)
+                          {
+                            char choix;
+                            PhyML_Printf("\n. The Ts/Tv ratio must be a positive number\n");
+                            PhyML_Printf("\n. Type any key to exit.\n");
+                            if(!scanf("%c",&choix)) Exit("\n");
+                            Exit("\n");
+                          }
+                        else
+                          {
+                            io->mod->kappa->v = (phydbl)atof(optarg);
+                          }
+                      }
+                  }
 	      }
 	    break;
 	  }
@@ -1425,6 +1514,17 @@ int Read_Command_Line(option *io, int argc, char **argv)
       /* io->mod->s_opt->opt_bl   = NO; */
     }
 
+  
+  if(io->do_tbe == YES)
+    {
+      io->do_alrt = NO;
+      io->do_boot = NO;
+    }
+  else
+    {
+      if(io->do_alrt == NO && io->n_boot_replicates > 0) io->do_boot = YES;        
+    }
+  
 #ifndef PHYML
   if((open_ps_file) || (io->m4_model == YES))
     {
@@ -1513,7 +1613,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
       io->fp_out_trees = Openfile(io->out_trees_file,1);
     }
   
-  if((io->print_boot_trees) && (io->mod->bootstrap > 0) && (io->fp_in_align != NULL))
+  if((io->print_boot_trees) && (io->do_boot == YES) && (io->fp_in_align != NULL))
     {
       strcpy(io->out_boot_tree_file,io->in_align_file);
       strcat(io->out_boot_tree_file,"_phyml_boot_trees");
@@ -1542,9 +1642,15 @@ int Read_Command_Line(option *io, int argc, char **argv)
       strcat(io->out_stats_file,".txt");
     }
 
+
+#ifdef PHYREX
+  strcat(io->out_summary_file,".txt");
+#endif
+
+  
   if(io->mod->ras->n_catg == 1) io->mod->s_opt->opt_alpha = 0;
   
-  if(!io->mod->s_opt->opt_subst_param)
+  if(io->mod->s_opt->opt_subst_param == NO)
     {
       io->mod->s_opt->opt_alpha  = NO;
       io->mod->s_opt->opt_kappa  = NO;
@@ -1558,7 +1664,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
      io->mod->whichmodel != F84 &&
      io->mod->whichmodel != TN93)
     {
-      io->mod->s_opt->opt_kappa = 0;
+      io->mod->s_opt->opt_kappa = NO;
     }
   
   if(io->datatype == AA && io->mod->whichmodel == CUSTOMAA && !io->mod->fp_aa_rate_mat)
@@ -1582,19 +1688,18 @@ int Read_Command_Line(option *io, int argc, char **argv)
       io->fp_out_stats = Openfile(io->out_stats_file,writemode);
     }
 #endif
-  
 
 #if defined(PHYREX)
   if(io->fp_in_align != NULL) io->fp_out_summary = Openfile(io->out_summary_file,writemode);
 #endif
-  
-  if(io->mod->whichmodel == GTR) 
+
+
+    if(io->quiet == NO)
     {
-      /* Make_Custom_Model(io->mod); */
-      io->mod->s_opt->opt_rr = 1;
+      PhyML_Printf("\n\n. Command line: ");
+      for(i=0;i<argc;i++) PhyML_Printf("%s ",argv[i]);
     }
-  
-  
+
   return 1;
 }
 
