@@ -52,7 +52,6 @@ void Free_Eigen(eigen *eigen_struct);
 void Free_One_Spr(t_spr *this_spr);
 void Free_Spr_List_One_Edge(t_tree *tree);
 void Free_Spr_List_All_Edge(t_tree *mixt_tree);
-void Free_Triplet(triplet *t);
 void Free_Actual_CSeq(calign *data);
 void Free_Prefix_Tree(pnode *n,int size);
 void Free_Pnode(pnode *n);
@@ -96,4 +95,5 @@ void Free_TBE_Matrices(int n_otu,  short unsigned*** i_matrix, short unsigned***
 		       short unsigned*** hamming, short unsigned** min_dist,
 		       short unsigned**  min_dist_edge, int** cluster_sizes);
 void Free_Extra_Edge_Lk(t_tree *tree);
+void Free_Edge_Length(t_edge *b);
 #endif
