@@ -133,42 +133,42 @@ void Launch_Interface(option *io)
   if(io->print_site_lnl)
     {
       strcpy(io->out_lk_file,io->in_align_file);
-      strcat(io->out_lk_file, "_phyml_lk");
+      strcat(io->out_lk_file, "_phyml_lk.txt");
+
       if(io->append_run_ID) { strcat(io->out_lk_file,"_"); strcat(io->out_lk_file,io->run_id_string); }
-      strcat(io->out_lk_file, ".txt");
       io->fp_out_lk = Openfile(io->out_lk_file,1);
     }
   
   if(io->print_trace)
     {
       strcpy(io->out_trace_file,io->in_align_file);
-      strcat(io->out_trace_file,"_phyml_trace");
+      strcat(io->out_trace_file,"_phyml_trace.txt");
+
       if(io->append_run_ID) { strcat(io->out_trace_file,"_"); strcat(io->out_trace_file,io->run_id_string); }
-      strcat(io->out_trace_file,".txt");
       io->fp_out_trace = Openfile(io->out_trace_file,1);
     }
   
   if(io->mod->s_opt->random_input_tree)
     {
       strcpy(io->out_trees_file,io->in_align_file);
-      strcat(io->out_trees_file,"_phyml_trees");
+      strcat(io->out_trees_file,"_phyml_trees.txt");
+
       if(io->append_run_ID) { strcat(io->out_trees_file,"_"); strcat(io->out_trees_file,io->run_id_string); }
-      strcat(io->out_trees_file,".txt");
       io->fp_out_trees = Openfile(io->out_trees_file,1);
     }
 
-  if((io->print_boot_trees) && (io->mod->bootstrap > 0))
+  if((io->print_boot_trees) && (io->n_boot_replicates > 0))
     {
       strcpy(io->out_boot_tree_file,io->in_align_file);
-      strcat(io->out_boot_tree_file,"_phyml_boot_trees");
+      strcat(io->out_boot_tree_file,"_phyml_boot_trees.txt");
+
       if(io->append_run_ID) { strcat(io->out_boot_tree_file,"_"); strcat(io->out_boot_tree_file,io->run_id_string); }
-      strcat(io->out_boot_tree_file,".txt");
       io->fp_out_boot_tree = Openfile(io->out_boot_tree_file,1);
       
       strcpy(io->out_boot_stats_file,io->in_align_file);
-      strcat(io->out_boot_stats_file,"_phyml_boot_stats");
+      strcat(io->out_boot_stats_file,"_phyml_boot_stats.txt");
+
       if(io->append_run_ID) { strcat(io->out_boot_stats_file,"_"); strcat(io->out_boot_stats_file,io->run_id_string); }
-      strcat(io->out_boot_stats_file,".txt");
       io->fp_out_boot_stats = Openfile(io->out_boot_stats_file,1);
     }
   
@@ -178,8 +178,6 @@ void Launch_Interface(option *io)
       strcat(io->out_stats_file,"_");
       strcat(io->out_tree_file,io->run_id_string);
       strcat(io->out_stats_file,io->run_id_string);
-      strcat(io->out_tree_file,".txt");
-      strcat(io->out_stats_file,".txt");
     }
 
   if(io->mod->ras->n_catg == 1) io->mod->s_opt->opt_alpha = 0;
@@ -195,11 +193,14 @@ void Launch_Interface(option *io)
   io->fp_out_tree  = Openfile(io->out_tree_file,1);
   io->fp_out_stats = Openfile(io->out_stats_file,1);
 
-  if(io->mod->whichmodel == GTR) 
-    {
-      /* Make_Custom_Model(io->mod); */
-      io->mod->s_opt->opt_rr = 1;
-    }
+// VINCENT: do not necessarily optimize relative rate parameters, they could be set by user
+	if(io->mod->whichmodel == GTR)
+	{
+		if (io->mod->s_opt->opt_subst_param)
+			io->mod->s_opt->opt_rr = YES;
+		else
+			io->mod->s_opt->opt_rr = NO;
+	}
 }
 
 //////////////////////////////////////////////////////////////
@@ -218,7 +219,6 @@ void Clear()
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 void Launch_Interface_Input(option *io)
 {
   char choix;
@@ -281,34 +281,22 @@ void Launch_Interface_Input(option *io)
 #if defined(PHYML) || defined(PART) || defined(PHYML_INSERT)
 
   strcpy(io->out_stats_file,io->in_align_file);
-  strcat(io->out_stats_file,"_phyml_stats.txt");
   
+  strcat(io->out_stats_file,"_phyml_stats.txt");  
   strcpy(io->out_tree_file,io->in_align_file);
+ 
   strcat(io->out_tree_file,"_phyml_tree.txt");
-
   strcpy(io->out_lk_file,io->in_align_file);
+
   strcat(io->out_lk_file,"_phyml_lk.txt");
-
-
 #endif
 
-
-#ifdef WIN32
 #ifdef EVOLVE
   if(Filexists("evolve_out.txt"));
 #elif OPTIMIZ
   if(Filexists("optimiz_out.txt"))
 #elif defined(PHYML) || defined(PART) || defined(PHYML_INSERT)
   if(Filexists(io->out_stats_file))
-#endif
-#elif UNIX
-#ifdef EVOLVE
-  if(Filexists("evolve_out"));
-#elif OPTIMIZ
-  if(Filexists("optimiz_out"))
-#elif defined(PHYML) || defined(PART) || defined(PHYML_INSERT)
-   if(Filexists(io->out_stats_file))
-#endif
 #endif
     {
       PhyML_Printf("\n");
@@ -337,22 +325,12 @@ void Launch_Interface_Input(option *io)
 
   /* io->fp_out_stats = Openfile(io->out_stats_file,io->out_stats_file_open_mode); */
 
-#ifdef WIN32
 #ifdef EVOLVE
   if(Filexists("evolve_seq.txt"))
 #elif OPTIMIZ
   if(Filexists("optimiz_tree.txt"))
 #elif defined(PHYML) || defined(PART) || defined(PHYML_INSERT)
   if(Filexists(io->out_tree_file))
-#endif
-#elif UNIX
-#ifdef EVOLVE
-  if(Filexists("evolve_seq"))
-#elif OPTIMIZ
-  if(Filexists("optimiz_tree"))
-#elif defined(PHYML) || defined(PART) || defined(PHYML_INSERT)
-  if(Filexists(io->out_tree_file))
-#endif
 #endif
     {
       PhyML_Printf("\n");
@@ -498,7 +476,7 @@ void Launch_Interface_Data_Type(option *io)
 	  }
 	#endif
 
-	if((io->mod->bootstrap > 1) && (io->n_data_sets > 1))
+	if((io->do_boot || io->do_tbe) && (io->n_data_sets > 1))
 	  {
 	    PhyML_Printf("\n. Bootstrap option is not allowed with multiple data sets !\n");
 	    PhyML_Printf("\n. Type any key to exit.");
@@ -638,7 +616,7 @@ void Launch_Interface_Model(option *io)
 		  PhyML_Printf("                [E] "
 			 "......... Equilibrium frequencies (empirical/user) "
 			 " %-15s \n",
-			 (io->mod->s_opt->user_state_freq)?("user defined"):("empirical"));
+			 (io->mod->e_frq->user_state_freq)?("user defined"):("empirical"));
 		}
 	      PhyML_Printf("                [K] "
 		     "............................. Current custom model "
@@ -799,7 +777,7 @@ void Launch_Interface_Model(option *io)
 	if(io->mod->whichmodel == CUSTOM)
 	  {
 	    rr_param = (char **)mCalloc(6,sizeof(char *));
-	    For(i,6) rr_param[i] = (char *)mCalloc(10,sizeof(char));
+	    for(i=0;i<6;i++) rr_param[i] = (char *)mCalloc(10,sizeof(char));
 	    rr = (char *)mCalloc(50,sizeof(char));
 
 	    mod = io->mod;
@@ -812,7 +790,7 @@ void Launch_Interface_Model(option *io)
 		Getstring_Stdin(io->mod->custom_mod_string->s);
 		if(strlen(io->mod->custom_mod_string->s) == 6)
 		  {
-		    For(i,6)
+		    for(i=0;i<6;i++)
 		      {
 			while(!isdigit((int)io->mod->custom_mod_string->s[i]))
 			  {
@@ -848,10 +826,10 @@ void Launch_Interface_Model(option *io)
 	    strcpy(rr_param[5],"G<->T");
 
 	    PhyML_Printf("\n. Set the relative rate values\n");
-	    For(i,mod->r_mat->n_diff_rr)
+	    for(i=0;i<mod->r_mat->n_diff_rr;i++)
 	      {
 		sprintf(rr,"\n. [");
-		For(j,6)
+		for(j=0;j<6;j++)
 		  {
 		    if(mod->r_mat->rr_num->v[j] == i) 
 		      {
@@ -880,7 +858,7 @@ void Launch_Interface_Model(option *io)
 		  }
 	      }
 
-	    For(i,6) Free(rr_param[i]);
+	    for(i=0;i<6;i++) Free(rr_param[i]);
 	    Free(rr_param);
 	    Free(rr);
 	  }
@@ -892,9 +870,9 @@ void Launch_Interface_Model(option *io)
 
 	if(io->mod->whichmodel == CUSTOM)
 	  {
-	    io->mod->s_opt->user_state_freq = (io->mod->s_opt->user_state_freq)?(0):(1);
+	    io->mod->e_frq->user_state_freq = (io->mod->e_frq->user_state_freq)?(0):(1);
 
-	    if(io->mod->s_opt->user_state_freq)
+	    if(io->mod->e_frq->user_state_freq)
 	      {
 		if(!io->mod->s_opt->opt_state_freq)
 		  {
@@ -904,7 +882,7 @@ void Launch_Interface_Model(option *io)
 		    int n_trial;
 		    
 		    bases = (char **)mCalloc(4,sizeof(char *));
-		    For(i,4) bases[i] = (char *)mCalloc(50,sizeof(char));
+		    for(i=0;i<4;i++) bases[i] = (char *)mCalloc(50,sizeof(char));
 		    bs = (char *)mCalloc(100,sizeof(char));
 		    
 		    strcpy(bases[0],". f(A)> ");
@@ -914,7 +892,7 @@ void Launch_Interface_Model(option *io)
 		    
 		    PhyML_Printf("\n. Set nucleotide frequencies \n");
 		    sum = .0;
-		    For(i,4)
+		    for(i=0;i<4;i++)
 		      {
 			PhyML_Printf("%s",bases[i]);
 			Getstring_Stdin(bs);
@@ -928,25 +906,25 @@ void Launch_Interface_Model(option *io)
 			    PhyML_Printf("\n. Enter a new value > ");
 			    Getstring_Stdin(bs);
 			  }
-			io->mod->user_b_freq->v[i] = (phydbl)atof(bs);
-			sum += io->mod->user_b_freq->v[i];
+			io->mod->e_frq->user_b_freq->v[i] = (phydbl)atof(bs);
+			sum += io->mod->e_frq->user_b_freq->v[i];
 		      }
 		
-		    For(i,4) io->mod->user_b_freq->v[i] /= sum;
+		    for(i=0;i<4;i++) io->mod->e_frq->user_b_freq->v[i] /= sum;
 
 		    if(sum > 1.0 || sum < 1.0)
 		      {
 			PhyML_Printf("\n. The nucleotide frequencies have to be normalised in order to sum to 1.0.\n");
 			PhyML_Printf("\n. The frequencies are now : f(A)=%f, f(C)=%f, f(G)=%f, f(T)=%f.\n",
-			       io->mod->user_b_freq->v[0],
-			       io->mod->user_b_freq->v[1],
-			       io->mod->user_b_freq->v[2],
-			       io->mod->user_b_freq->v[3]);			  
+			       io->mod->e_frq->user_b_freq->v[0],
+			       io->mod->e_frq->user_b_freq->v[1],
+			       io->mod->e_frq->user_b_freq->v[2],
+			       io->mod->e_frq->user_b_freq->v[3]);			  
 			PhyML_Printf("\n. Enter any key to continue.\n");
 			if(!scanf("%c",bs)) Exit("\n");
 		      }
 
-		    For(i,4) Free(bases[i]);
+		    for(i=0;i<4;i++) Free(bases[i]);
 		    Free(bases);
 		    Free(bs);
 		  }
@@ -1651,10 +1629,11 @@ void Launch_Interface_Branch_Support(option *io)
   PhyML_Printf("\n");
 
 
-  strcpy(s,(io->mod->bootstrap > 0)?("yes"):("no"));
-  if(io->mod->bootstrap > 0) sprintf(s+strlen(s)," (%d replicate%s)",
-					io->mod->bootstrap,
-					(io->mod->bootstrap>1)?("s"):(""));
+  strcpy(s,(io->do_boot || io->do_tbe)?("yes"):("no"));
+  if(io->n_boot_replicates > 0) sprintf(s+strlen(s)," (%d replicate%s%s)",
+					io->n_boot_replicates,
+					(io->n_boot_replicates>1)?("s"):(""),
+					(io->do_tbe)?(", TBE"):(""));
   
   /*   PhyML_Printf("                [+] " */
   PhyML_Printf("                [B] "
@@ -1713,7 +1692,7 @@ void Launch_Interface_Branch_Support(option *io)
 
     case 'B' :
       {
-	if(io->mod->bootstrap > 0) io->mod->bootstrap = 0;
+	if(io->n_boot_replicates > 0) io->n_boot_replicates = 0;
 	else
 	  {
 	    char *r;
@@ -1743,7 +1722,7 @@ void Launch_Interface_Branch_Support(option *io)
 		PhyML_Printf("\n. Enter a new value > ");
 		Getstring_Stdin(r);
 	      }
-	    io->mod->bootstrap = atoi(r);
+	    io->n_boot_replicates = atoi(r);
 
 	    PhyML_Printf("\n. Print bootstrap trees (and statistics) ? (%s) > ",
                          (io->print_boot_trees)?("Y/n"):("y/N"));
@@ -1759,10 +1738,11 @@ void Launch_Interface_Branch_Support(option *io)
 		  io->print_boot_trees = 1;
 
 		  strcpy(io->out_boot_tree_file,io->in_align_file);
+
 		  strcat(io->out_boot_tree_file,"_phyml_boot_trees.txt");
 		  io->fp_out_boot_tree = Openfile(io->out_boot_tree_file,1);
-
 		  strcpy(io->out_boot_stats_file,io->in_align_file);
+
 		  strcat(io->out_boot_stats_file,"_phyml_boot_stats.txt");
 		  io->fp_out_boot_stats = Openfile(io->out_boot_stats_file,1);
 
@@ -1776,13 +1756,40 @@ void Launch_Interface_Branch_Support(option *io)
 		  break;
 		}
 	      }
+
+	    PhyML_Printf("\n. Compute TBE instead of FBP ? (%s) > ",
+                         (io->do_tbe)?("Y/n"):("y/N"));
+
+	    if(!scanf("%c",&answer)) Exit("\n");
+	    if(answer == '\n') answer = (io->do_tbe)?('Y'):('N');
+	    else getchar();
+
+	    switch(answer)
+	      {
+	      case 'Y' : case 'y' :
+		{
+		  io->do_tbe = YES;
+		  io->do_boot = NO;
+		  break;
+		}
+	      case 'N' : case 'n' :
+		{
+		  io->do_tbe = NO;
+		  io->do_boot = YES;
+		  break;
+		}
+	      }
 	    Free(r);
 	  }
 	break;
       }
     case 'A' :
       {
-	io->mod->bootstrap = 0;
+        io->do_boot = NO;
+        io->do_tbe  = NO;
+        io->do_alrt = YES;
+        
+	io->n_boot_replicates = 0;
 
 	switch(io->ratio_test)
 	  {
@@ -1857,7 +1864,7 @@ void Launch_Interface_Multigene(option *io)
       io->st       = (supert_tree *)PART_Make_Supert_tree_Light(io);
       io->st->n_part = io->n_data_sets;
 
-      For(set,io->n_part)
+      for(set=0;set<io->n_part;set++)
 	{
 	  io->st->optionlist[set] = Make_Input();
 	  Set_Defaults_Input(io->st->optionlist[set]);
