@@ -20,15 +20,14 @@ the GNU public licence.  See http://www.opensource.org for details.
 t_tree *Read_Tree(char **s_tree);
 void R_rtree(char *s_tree_a,char *s_tree_d,t_node *a,t_tree *tree,int *n_int,int *n_ext);
 void Read_Branch_Label(char *s_d,char *s_a,t_edge *b);
-void Read_Branch_Length(char *s_d,char *s_a,t_tree *tree);
-void Read_Node_Name(t_node *d,char *s_tree_d,t_tree *tree);
+void Read_Branch_Length(char *s_d, char *s_a, t_edge *b, t_tree *tree);
+void Read_Node_Name(t_node *d, char *s_tree_d, t_edge *b, t_tree *tree);
 void Clean_Multifurcation(char **subtrees,int current_deg,int end_deg);
 char **Sub_Trees(char *tree,int *degree);
 int Next_Par(char *s,int pos);
 void Print_List(t_ll *list);
 void Print_Tree(FILE *fp,t_tree *tree);
 char *Write_Tree(t_tree *tree,int custom);
-void R_wtree(t_node *pere,t_node *fils,int *available,char **s_tree,t_tree *tree);
 void R_wtree_Custom(t_node *pere,t_node *fils,int *available,char **s_tree,int *pos,t_tree *tree);
 void Detect_Align_File_Format(option *io);
 void Detect_Tree_File_Format(option *io);
@@ -51,7 +50,8 @@ void Print_Node(t_node *a,t_node *d,t_tree *tree);
 void Print_Model(t_mod *mod);
 void Print_Mat(matrix *mat);
 FILE *Openfile(char *filename,int mode);
-void Print_Fp_Out(FILE *fp_out,time_t t_beg,time_t t_end,t_tree *tree,option *io,int n_data_set,int num_tree, int add_citation);
+//void Print_Fp_Out(FILE *fp_out,time_t t_beg,time_t t_end,t_tree *tree,option *io,int n_data_set,int num_tree, int add_citation);
+void Print_Fp_Out(FILE *fp_out,time_t t_beg,time_t t_end,t_tree *tree,option *io,int n_data_set,int num_tree, int add_citation, int precision);
 void Print_Fp_Out_Lines(FILE *fp_out,time_t t_beg,time_t t_end,t_tree *tree,option *io,int n_data_set);
 void Print_Freq(t_tree *tree);
 void Print_Settings(option *io);
@@ -102,5 +102,9 @@ void JSON_Tree_Io(t_tree *tree, FILE *where);
 json_o *JSON_Tree_To_Object(t_tree *tree);
 json_o *JSON_Tree_To_Object_Light(t_tree *mixt_tree);
 scalar_dbl *Read_Io_Weights(option *io);
+void Print_Lk_Given_Edge_Recurr(t_node *a, t_node *d, t_edge *b, t_tree *tree);
+void R_wtree(t_node *pere, t_node *fils, t_edge *b, int *available, char **s_tree, t_tree *tree);
+void Read_Branch_Support(char *s_d, char *s_a, t_edge *b, t_tree *tree);
+void Collect_Edge_Support_Values(t_tree *tree);
 
 #endif
