@@ -24,7 +24,6 @@ the GNU public licence. See http://www.opensource.org for details.
 #include "alrt.h"
 #include "mixt.h"
 #include "invitee.h"
-//#include "geo.h"
 
 #ifdef MPI
 #include "mpi_boot.h"
@@ -58,20 +57,23 @@ int main(int argc, char **argv)
   rc = MPI_Init(&argc,&argv);
   if (rc != MPI_SUCCESS) 
     {
-      PhyML_Printf("\n== Err. starting MPI program. Terminating.\n");
+      PhyML_Fprintf(stderr,"\n. Err. starting MPI program. Terminating.\n");
       MPI_Abort(MPI_COMM_WORLD, rc);
     }
-  MPI_Comm_size(MPI_COMM_WORLD,&Global_numTask);
-  MPI_Comm_rank(MPI_COMM_WORLD,&Global_myRank);
+  if(MPI_Comm_size(MPI_COMM_WORLD,&Global_numTask) != MPI_SUCCESS) MPI_Abort(MPI_COMM_WORLD, rc);
+  if(MPI_Comm_rank(MPI_COMM_WORLD,&Global_myRank) != MPI_SUCCESS) MPI_Abort(MPI_COMM_WORLD, rc);
+  PhyML_Fprintf(stdout,"\n\n. Running the analysis on %d CPU%s.",
+                Global_numTask,
+                Global_numTask>1?"s.":".");
 #endif
 
 #ifdef QUIET
   setvbuf(stdout,NULL,_IOFBF,2048);
 #endif
 
-  tree             = NULL;
-  mod              = NULL;
-  best_lnL         = UNLIKELY;
+  tree     = NULL;
+  mod      = NULL;
+  best_lnL = UNLIKELY;
 
 
   io = (option *)Get_Input(argc,argv);
@@ -81,7 +83,7 @@ int main(int argc, char **argv)
       Free(io);
       return(0);
     }
-
+  
 #ifdef EVOLVE
   io->colalias = NO;
 #endif
@@ -142,7 +144,6 @@ int main(int argc, char **argv)
 
                   if(io->mod->use_m4mod) M4_Init_Model(mod->m4mod,cdata,mod);
 
-                  Remove_Duplicates(cdata,mod,io);
 
                   //Make the initial tree
                   switch(io->in_tree)
@@ -151,6 +152,8 @@ int main(int argc, char **argv)
                     case 2 :          { tree = Read_User_Tree(cdata,mod,io); break; }
                     }
 
+                  if(io->mod->s_opt->opt_topo == YES) Remove_Duplicates(cdata,io,tree);
+
                   if(io->fp_in_constraint_tree != NULL)
                     {
                       char *s;
@@ -180,7 +183,6 @@ int main(int argc, char **argv)
                     }
 
                   if(!tree) continue;
-
                   
                   time(&t_beg);
                   time(&(tree->t_beg));
@@ -191,6 +193,7 @@ int main(int argc, char **argv)
                   tree->n_pattern    = tree->data->crunch_len;
                   tree->n_root       = NULL;
                   tree->e_root       = NULL;
+                  tree->n_tot_bl_opt = 0;
 
                   Set_Both_Sides(YES,tree);
 
@@ -209,7 +212,10 @@ int main(int argc, char **argv)
                       Exit("\n");
                     }
 
-                  Prepare_Tree_For_Lk(tree);
+                  Connect_CSeqs_To_Nodes(tree->data,tree->io,tree);
+                  Make_Tree_For_Pars(tree);
+                  Make_Tree_For_Lk(tree);
+                  Make_Spr(tree);
                   Br_Len_Not_Involving_Invar(tree);
                   Unscale_Br_Len_Multiplier_Tree(tree);
 
@@ -229,20 +235,17 @@ int main(int argc, char **argv)
 
 #ifdef PHYML
                   if(tree->io->print_json_trace == YES) JSON_Tree_Io(tree,tree->io->fp_out_json_trace); 
-                                                     
+
 
                   
-                  Switch_Eigen(YES,tree->mod);                 
+                  Set_Update_Eigen(YES,tree->mod);                 
                   Lk(NULL,tree);
-                  Switch_Eigen(NO,tree->mod);
-
-
-                  tree->n_tot_bl_opt = 0;
-
+                  Set_Update_Eigen(NO,tree->mod);
+                  
+                  
                   if(tree->mod->s_opt->opt_topo)
 		    {
-                      Global_Spr_Search(tree);
-                      
+                      Global_Spr_Search(tree);                      
                       if(tree->n_root) Add_Root(tree->a_edges[0],tree);
                     }
                   else
@@ -250,10 +253,7 @@ int main(int argc, char **argv)
 #ifdef BEAGLE
                       tree->b_inst = create_beagle_instance(tree, io->quiet, io);
 #endif
-                      //Optimize Branch lengths?
-                      /* if(tree->mod->s_opt->opt_subst_param || tree->mod->s_opt->opt_bl)  */
-                        Round_Optimize(tree,ROUND_MAX);
-                       
+                      if(tree->mod->s_opt->opt_subst_param || tree->mod->s_opt->opt_bl) Round_Optimize(tree,ROUND_MAX);                      
                     }
 
                   
@@ -265,17 +265,14 @@ int main(int argc, char **argv)
                   Get_Tree_Size(tree);
                   PhyML_Printf("\n\n. Log likelihood of the current tree: %f.",tree->c_lnL);
 
-                          
+                  
                   if(tree->io->ancestral == YES) Ancestral_Sequences(tree,YES);
                   
-                  /* Build_Distrib_Number_Of_Diff_States_Under_Model(tree); */
-                  /* Exit("\n"); */
-
-
                   Check_Br_Lens(tree);
                   Br_Len_Involving_Invar(tree);
                   Rescale_Br_Len_Multiplier_Tree(tree);
-
+                  
+                  
 #elif defined EVOLVE
                   Evolve(tree->data,tree->mod,tree);
                   Exit("\n. Exiting 'evolve'\n");
@@ -297,6 +294,7 @@ int main(int argc, char **argv)
                       best_lnL = tree->c_lnL;
                       if(most_likely_tree) Free(most_likely_tree);
                       most_likely_tree = Write_Tree(tree,NO);
+                      
                     }
 
                   time(&t_end);
@@ -304,7 +302,7 @@ int main(int argc, char **argv)
                   Print_Fp_Out(io->fp_out_stats,t_beg,t_end,tree,
                                io,num_data_set+1,
                                (orig_random_input_tree == YES)?(num_rand_tree):(num_tree),
-                               (num_rand_tree == io->mod->s_opt->n_rand_starts-1)?(YES):(NO));
+                               (num_rand_tree == io->mod->s_opt->n_rand_starts-1)?(YES):(NO), io->precision);
 
                   if(tree->io->print_site_lnl) Print_Site_Lk(tree,io->fp_out_lk);
 
@@ -321,7 +319,6 @@ int main(int argc, char **argv)
                   Free_One_Spr(tree->best_spr);
                   Free_Spr_List_One_Edge(tree);
                   Free_Spr_List_All_Edge(tree);
-                  Free_Triplet(tree->triplet_struct);
                   Free_Tree_Pars(tree);
                   Free_Tree_Lk(tree);
                   Free_Tree(tree);
@@ -332,13 +329,13 @@ int main(int argc, char **argv)
 
 
               /* Launch bootstrap analysis */
-              if(mod->bootstrap)
+              if(io->do_boot || io->do_tbe)
                 {
                   if(!io->quiet) PhyML_Printf("\n\n. Launch bootstrap analysis on the most likely tree...");
 
 #ifdef MPI
                   MPI_Bcast (most_likely_tree, strlen(most_likely_tree)+1, MPI_CHAR, 0, MPI_COMM_WORLD);
-                  if(!io->quiet)  PhyML_Printf("\n\n. The bootstrap analysis will use %d CPU%c.",Global_numTask,Global_numTask>1?'s':'\0');
+                  if(!io->quiet) PhyML_Printf("\n\n. The bootstrap analysis will use %d CPU%c.",Global_numTask,Global_numTask>1?'s':'\0');
 #endif
                   
                   most_likely_tree = Bootstrap_From_String(most_likely_tree,cdata,mod,io);
@@ -352,9 +349,22 @@ int main(int argc, char **argv)
                     most_likely_tree = aLRT_From_String(most_likely_tree,cdata,mod,io);
                   }
 
+
               /* Print the most likely tree in the output file */
               if(!io->quiet) PhyML_Printf("\n\n. Printing the most likely tree in file '%s'.", Basename(io->out_tree_file));
               if(io->n_data_sets == 1) rewind(io->fp_out_tree);
+              
+              t_tree *dum;
+              dum = Read_Tree(&most_likely_tree);
+              dum->data = cdata;
+              dum->mod  = mod;
+              dum->io   = io;
+              Connect_CSeqs_To_Nodes(cdata,io,dum);
+              Insert_Duplicates(dum);
+              Free(most_likely_tree);
+              most_likely_tree = Write_Tree(dum,NO);
+              Free_Tree(dum);
+              
               PhyML_Fprintf(io->fp_out_tree,"%s\n",most_likely_tree);
 
               if(io->n_trees > 1 && io->n_data_sets > 1) break;
@@ -436,24 +446,39 @@ int main(int argc, char **argv)
 #elif(RF)
 int main(int argc, char **argv)
 {
-  t_tree *tree1, *tree2;
-  FILE *fp_tree1, *fp_tree2;
-  int i,j;
-
-  fp_tree1 = (FILE *)fopen(argv[1],"r");
-  fp_tree2 = (FILE *)fopen(argv[2],"r");
-
-  tree1 = Read_Tree_File_Phylip(fp_tree1);
-  tree2 = Read_Tree_File_Phylip(fp_tree2);
+  option *io;
+  int r_seed;
 
 
-  Prune_Tree(tree1,tree2);
-  Prune_Tree(tree2,tree1);
+  io = (option *)Get_Input(argc,argv);
+  if(!io) return(0);
+
+  r_seed = (io->r_seed < 0)?(time(NULL)):(io->r_seed);
+  srand(r_seed);
+  io->r_seed = r_seed;
+  
+  Get_Seq(io);
+  Shuffle_Sites(io->mod->ras->pinvar->v,io->data,io->n_otu);
+  Print_Seq(stdout,io->data,io->n_otu);
+  
+  /* t_tree *tree1, *tree2; */
+  /* FILE *fp_tree1, *fp_tree2; */
+  /* int i,j; */
+
+  /* fp_tree1 = (FILE *)fopen(argv[1],"r"); */
+  /* fp_tree2 = (FILE *)fopen(argv[2],"r"); */
+
+  /* tree1 = Read_Tree_File_Phylip(fp_tree1); */
+  /* tree2 = Read_Tree_File_Phylip(fp_tree2); */
+
+
+  /* Prune_Tree(tree1,tree2); */
+  /* Prune_Tree(tree2,tree1); */
   
 
-  PhyML_Printf("%s\n%s",
-               Write_Tree(tree1,NO),
-               Write_Tree(tree2,NO));
+  /* PhyML_Printf("%s\n%s", */
+  /*              Write_Tree(tree1,NO), */
+  /*              Write_Tree(tree2,NO)); */
                
   
   /* Match_Nodes_In_Small_Tree(tree1,tree2); */
@@ -560,6 +585,28 @@ int main(int argc, char **argv)
 #include "xml.h"
 int main(int argc, char **argv)
 {
+  option *io = Get_Input(argc,argv);
+  char *s = (char *)mCalloc(5,sizeof(char));
+  int i;
+  
+  Get_Seq(io);
+  
+  for(i=0;i<io->n_otu;++i)
+    {      
+      PhyML_Printf("\n\n\n");
+      PhyML_Printf("\n<clade id=\"clad%d\">",i+1);
+      PhyML_Printf("\n<taxon value=\"%s\"/>",io->data[i]->name);
+      PhyML_Printf("\n</clade>");
+      PhyML_Printf("\n");
+      PhyML_Printf("\n<calibration id=\"cal%d\">",i+1);
+      strncpy(s,io->data[i]->name,4);
+      s[4]='\0';
+      PhyML_Printf("\n<lower>%d</lower>",atoi(s)-2017);
+      PhyML_Printf("\n<upper>%d</upper>",atoi(s)-2017);
+      PhyML_Printf("\n<appliesto clade.id=\"clad%d\"/>",i+1);
+      PhyML_Printf("\n</calibration>");
+    }
+
 }
 
 #elif(INVITEE)
