@@ -14,10 +14,15 @@ the GNU public licence. See http://www.opensource.org for details.
 
 //////////////////////////////////////////////////////////////
 
-void Make_Tree_4_Lk(t_tree *tree, calign *cdata, int n_site)
+void Make_Tree_For_Lk(t_tree *tree)
 {
   int i;
+  calign *cdata;
 
+  cdata = tree->data;
+  assert(cdata);
+
+  
   tree->c_lnL_sorted         = (phydbl *)mCalloc(tree->n_pattern,sizeof(phydbl));
   tree->cur_site_lk          = (phydbl *)mCalloc(tree->n_pattern,sizeof(phydbl));
   tree->old_site_lk          = (phydbl *)mCalloc(tree->n_pattern,sizeof(phydbl));
@@ -25,12 +30,13 @@ void Make_Tree_4_Lk(t_tree *tree, calign *cdata, int n_site)
   tree->unscaled_site_lk_cat = (phydbl *)mCalloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->n_pattern,sizeof(phydbl));
   tree->fact_sum_scale       = (int *)mCalloc(tree->n_pattern,sizeof(int));
 
+  
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-  if(posix_memalign((void **)&tree->eigen_lr_left,BYTE_ALIGN,(size_t)MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
-  if(posix_memalign((void **)&tree->eigen_lr_rght,BYTE_ALIGN,(size_t)MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
-  if(posix_memalign((void **)&tree->dot_prod,BYTE_ALIGN,(size_t)tree->n_pattern*tree->mod->ns*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
-  if(posix_memalign((void **)&tree->expl,BYTE_ALIGN,(size_t)3*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->eigen_lr_left,BYTE_ALIGN,(size_t)MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->eigen_lr_rght,BYTE_ALIGN,(size_t)MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->dot_prod,BYTE_ALIGN,(size_t)tree->n_pattern*tree->mod->ns*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->expl,BYTE_ALIGN,(size_t)3*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
   tree->eigen_lr_left = _aligned_malloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
   tree->eigen_lr_rght = _aligned_malloc(MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
@@ -60,13 +66,31 @@ void Make_Tree_4_Lk(t_tree *tree, calign *cdata, int n_site)
 
       Init_Partial_Lk_Tips_Double(tree);
       Init_Partial_Lk_Loc(tree);
-
+      
       if(tree->n_root != NULL)
         {
           Free_Edge_Lk_Rght(tree->n_root->b[1]);
           Free_Edge_Lk_Rght(tree->n_root->b[2]);
           Free_Edge_Loc_Rght(tree->n_root->b[1]);
           Free_Edge_Loc_Rght(tree->n_root->b[2]);
+
+          tree->n_root->b[1]->p_lk_rght  = tree->e_root->p_lk_left;
+          tree->n_root->b[2]->p_lk_rght  = tree->e_root->p_lk_rght;
+          
+          tree->n_root->b[1]->p_lk_tip_r = tree->e_root->p_lk_tip_l;      
+          tree->n_root->b[2]->p_lk_tip_r = tree->e_root->p_lk_tip_r;
+          
+          tree->n_root->b[1]->div_post_pred_rght  = tree->e_root->div_post_pred_rght;
+          tree->n_root->b[2]->div_post_pred_rght  = tree->e_root->div_post_pred_left;
+          
+          tree->n_root->b[1]->sum_scale_rght  = tree->e_root->sum_scale_rght;
+          tree->n_root->b[2]->sum_scale_rght  = tree->e_root->sum_scale_left;
+          
+          tree->n_root->b[1]->sum_scale_rght_cat  = tree->e_root->sum_scale_rght_cat;
+          tree->n_root->b[2]->sum_scale_rght_cat  = tree->e_root->sum_scale_left_cat;
+          
+          tree->n_root->b[1]->patt_id_rght  = tree->e_root->patt_id_rght;
+          tree->n_root->b[2]->patt_id_rght  = tree->e_root->patt_id_left;
         }
     }
 }
@@ -74,14 +98,21 @@ void Make_Tree_4_Lk(t_tree *tree, calign *cdata, int n_site)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
-void Make_Tree_4_Pars(t_tree *tree, calign *cdata, int n_site)
+void Make_Tree_For_Pars(t_tree *tree)
 {
   int i;
+  calign *cdata;
+
+  cdata = tree->data;
+  assert(cdata);
+
+  assert(tree->mod);
+  
   tree->site_pars = (int *)mCalloc(tree->n_pattern,sizeof(int));
   tree->step_mat = (int *)mCalloc(tree->mod->ns * tree->mod->ns,sizeof(int));
 
-  For(i,2*tree->n_otu-1) Make_Edge_Pars(tree->a_edges[i],tree);
+  for(i=0;i<2*tree->n_otu-1;++i) Make_Edge_Pars(tree->a_edges[i],tree);
+
   Init_Ui_Tips(tree);
   Init_Partial_Pars_Tips(tree); /* Must be called after Init_Ui_Tips is called */
 
@@ -142,7 +173,7 @@ void Make_New_Edge_Label(t_edge *b)
 t_edge *Make_Edge_Light(t_node *a, t_node *d, int num)
 {
   t_edge *b;
-
+  
   b = (t_edge *)mCalloc(1,sizeof(t_edge));
 
   b->l = (scalar_dbl *)mCalloc(1,sizeof(scalar_dbl));
@@ -161,7 +192,8 @@ t_edge *Make_Edge_Light(t_node *a, t_node *d, int num)
 
   if(a && b)
     {
-      b->left = a;  b->rght = d;
+      b->left = a;
+      b->rght = d;
       if(a->tax) {b->rght = a; b->left = d;} /* root */
       /* a tip is necessary on the right side of the t_edge */
 
@@ -169,9 +201,10 @@ t_edge *Make_Edge_Light(t_node *a, t_node *d, int num)
         (Set_Edge_Dirs(b,a,d,NULL)):
         (Set_Edge_Dirs(b,d,a,NULL));
 
-      b->l->v             = a->l[b->l_r];
-      if(a->tax) b->l->v  = a->l[b->r_l];
-      b->l_old->v         = b->l->v;
+      assert(b->l_r > -1);
+      assert(b->r_l > -1);
+      
+      b->l_old->v = b->l->v;
     }
   else
     {
@@ -188,6 +221,7 @@ t_edge *Make_Edge_Light(t_node *a, t_node *d, int num)
 
 void Make_Edge_Pars(t_edge *b, t_tree *tree)
 {
+  assert(b);
   Make_Edge_Pars_Left(b,tree);
   Make_Edge_Pars_Rght(b,tree);
 }
@@ -239,7 +273,7 @@ void Make_Edge_Lk(t_edge *b, t_tree *tree)
   b->Pij_rr = (phydbl *)mCalloc(tree->mod->ras->n_catg*tree->mod->ns*tree->mod->ns,sizeof(phydbl));
   b->tPij_rr = (phydbl *)mCalloc(tree->mod->ras->n_catg*tree->mod->ns*tree->mod->ns,sizeof(phydbl));
 #endif
-
+  
   Make_Edge_Lk_Left(b,tree);
   Make_Edge_Lk_Rght(b,tree);
 }
@@ -266,7 +300,7 @@ void Make_Edge_Lk_Left(t_edge *b, t_tree *tree)
         {
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-          if(posix_memalign((void **)&b->p_lk_left,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+          if(posix_memalign((void **)&b->p_lk_left,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
           b->p_lk_left = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -281,7 +315,7 @@ void Make_Edge_Lk_Left(t_edge *b, t_tree *tree)
           
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-          if(posix_memalign((void **)&b->p_lk_tip_l,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+          if(posix_memalign((void **)&b->p_lk_tip_l,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
           b->p_lk_tip_l = _aligned_malloc(tree->data->crunch_len*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -302,7 +336,7 @@ void Make_Edge_Lk_Left(t_edge *b, t_tree *tree)
 
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-      if(posix_memalign((void **)&b->p_lk_left,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+      if(posix_memalign((void **)&b->p_lk_left,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
       b->p_lk_left = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -337,7 +371,7 @@ void Make_Edge_Lk_Rght(t_edge *b, t_tree *tree)
         {
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-	  if(posix_memalign((void **)&b->p_lk_rght,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+	  if(posix_memalign((void **)&b->p_lk_rght,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else          
           b->p_lk_rght = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -350,7 +384,7 @@ void Make_Edge_Lk_Rght(t_edge *b, t_tree *tree)
         {
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-          if(posix_memalign((void **)&b->p_lk_tip_r,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+          if(posix_memalign((void **)&b->p_lk_tip_r,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
           b->p_lk_tip_r = _aligned_malloc(tree->data->crunch_len*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -371,7 +405,7 @@ void Make_Edge_Lk_Rght(t_edge *b, t_tree *tree)
       b->sum_scale_rght = (int *)mCalloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes),sizeof(int));
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-      if(posix_memalign((void **)&b->p_lk_rght,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+      if(posix_memalign((void **)&b->p_lk_rght,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
       b->p_lk_rght      = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -400,7 +434,7 @@ void Make_Extra_Edge_Lk(t_tree *tree)
 
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-  if(posix_memalign((void **)&tree->p_lk_extra_0,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->p_lk_extra_0,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
   tree->p_lk_extra_0 = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -411,7 +445,7 @@ void Make_Extra_Edge_Lk(t_tree *tree)
   
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-  if(posix_memalign((void **)&tree->p_lk_tip_extra_0,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->p_lk_tip_extra_0,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
   tree->p_lk_tip_extra_0 = _aligned_malloc(tree->data->crunch_len*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -429,7 +463,7 @@ void Make_Extra_Edge_Lk(t_tree *tree)
 
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-  if(posix_memalign((void **)&tree->p_lk_extra_1,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->p_lk_extra_1,BYTE_ALIGN,(size_t)tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
   tree->p_lk_extra_1 = _aligned_malloc(tree->data->crunch_len*MAX(tree->mod->ras->n_catg,tree->mod->n_mixt_classes)*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -440,7 +474,7 @@ void Make_Extra_Edge_Lk(t_tree *tree)
   
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-  if(posix_memalign((void **)&tree->p_lk_tip_extra_1,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&tree->p_lk_tip_extra_1,BYTE_ALIGN,(size_t)tree->data->crunch_len*tree->mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
   tree->p_lk_tip_extra_1 = _aligned_malloc(tree->data->crunch_len*tree->mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -506,17 +540,16 @@ t_nni *Make_NNI()
 
 t_node *Make_Node_Light(int num)
 {
- t_node *n;
-
+  t_node *n;
+  
   n           = (t_node *)mCalloc(1,sizeof(t_node));
   n->v        = (t_node **)mCalloc(3,sizeof(t_node *));
-  n->l        = (phydbl *)mCalloc(3,sizeof(phydbl));
   n->b        = (t_edge **)mCalloc(3,sizeof(t_edge *));
   n->score    = (phydbl *)mCalloc(3,sizeof(phydbl));
   n->s_ingrp  = (int *)mCalloc(3,sizeof(int));
   n->s_outgrp = (int *)mCalloc(3,sizeof(int));
   n->cal      = (t_cal **)mCalloc(MAX_N_CAL,sizeof(t_cal *));
- 
+
   Init_Node_Light(n,num);
 
   return n;
@@ -525,14 +558,15 @@ t_node *Make_Node_Light(int num)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
-
 void Make_Node_Lk(t_node *n)
 {
 /*   n->n_ex_nodes = (int *)mCalloc(2,sizeof(int)); */
   return;
 }
 
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 nexcom **Make_Nexus_Com()
 {
   nexcom **com;
@@ -553,7 +587,6 @@ nexcom **Make_Nexus_Com()
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 nexparm *Make_Nexus_Parm()
 {
   nexparm *parm;
@@ -664,8 +697,8 @@ void Make_All_Tree_Nodes(t_tree *tree)
   for(i=0;i<2*tree->n_otu-1;++i)
     {
       tree->a_nodes[i] = (t_node *)Make_Node_Light(i);
-      if(i < tree->n_otu) tree->a_nodes[i]->tax = 1;
-      else                tree->a_nodes[i]->tax = 0;
+      if(i < tree->n_otu) tree->a_nodes[i]->tax = YES;
+      else                tree->a_nodes[i]->tax = NO;
     }
 }
 
@@ -682,7 +715,7 @@ void Make_All_Tree_Edges(t_tree *tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-calign *Make_Calign(int n_otu, int crunch_len, int state_len, int init_len, char **sp_names)
+calign *Make_Calign(int n_otu, int crunch_len, int state_len, int init_len, char **sp_names_in, int n_rm, char **sp_names_out)
 {
   calign *cdata;
   int j;
@@ -695,16 +728,28 @@ calign *Make_Calign(int n_otu, int crunch_len, int state_len, int init_len, char
   cdata->invar         = (short int *)mCalloc(crunch_len,sizeof(short int));
   cdata->sitepatt      = (int *)mCalloc(init_len,sizeof(int ));
 
+  if(n_rm > 0) cdata->c_seq_rm = (align **)mCalloc(n_rm,sizeof(align *));
+
   for(j=0;j<n_otu;j++)
     {
       cdata->c_seq[j]            = (align *)mCalloc(1,sizeof(align));
-      cdata->c_seq[j]->name      = (char *)mCalloc((int)(strlen(sp_names[j])+1),sizeof(char));
-      strcpy(cdata->c_seq[j]->name,sp_names[j]);
-      cdata->c_seq[j]->state     = (char *)mCalloc(crunch_len*state_len,sizeof(char));
+      cdata->c_seq[j]->name      = (char *)mCalloc((int)(strlen(sp_names_in[j])+1),sizeof(char));
+      strcpy(cdata->c_seq[j]->name,sp_names_in[j]);
+      cdata->c_seq[j]->state     = (char *)mCalloc(crunch_len*state_len+1,sizeof(char));
       cdata->c_seq[j]->d_state   = (int *)mCalloc(crunch_len*state_len,sizeof(int));
       cdata->c_seq[j]->is_ambigu = (short int *)mCalloc(crunch_len,sizeof(short int));
     }
 
+  for(j=0;j<n_rm;j++)
+    {
+      cdata->c_seq_rm[j]            = (align *)mCalloc(1,sizeof(align));
+      cdata->c_seq_rm[j]->name      = (char *)mCalloc((int)(strlen(sp_names_out[j])+1),sizeof(char));
+      strcpy(cdata->c_seq_rm[j]->name,sp_names_out[j]);
+      cdata->c_seq_rm[j]->state     = (char *)mCalloc(crunch_len*state_len+1,sizeof(char));
+      cdata->c_seq_rm[j]->d_state   = (int *)mCalloc(crunch_len*state_len,sizeof(int));
+      cdata->c_seq_rm[j]->is_ambigu = (short int *)mCalloc(crunch_len,sizeof(short int));
+    }
+  
   return cdata;
 }
 
@@ -833,7 +878,7 @@ void Make_Model_Complete(t_mod *mod)
 
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-  if(posix_memalign((void **)&mod->Pij_rr->v,BYTE_ALIGN,(size_t)mod->ras->n_catg*mod->ns*mod->ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&mod->Pij_rr->v,BYTE_ALIGN,(size_t)mod->ras->n_catg*mod->ns*mod->ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
   mod->Pij_rr->v = _aligned_malloc(mod->ras->n_catg*mod->ns*mod->ns*sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -949,7 +994,7 @@ t_efrq *Make_Efrq(int ns)
 
 #if (defined(__AVX__) || defined(__SSE3__))
 #ifndef WIN32
-  if(posix_memalign((void **)&e_frq->pi->v,BYTE_ALIGN,(size_t)ns*sizeof(double))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&e_frq->pi->v,BYTE_ALIGN,(size_t)ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
 #else
   e_frq->pi->v = _aligned_malloc(ns * sizeof(phydbl),BYTE_ALIGN);
 #endif
@@ -1017,7 +1062,8 @@ option *Make_Input()
   io->out_file                          = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_tree_file                     = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_trees_file                    = (char *)mCalloc(T_MAX_FILE,sizeof(char));
-  io->out_ancestral_file                = (char *)mCalloc(T_MAX_FILE,sizeof(char));
+  io->out_ancestral_seq_file            = (char *)mCalloc(T_MAX_FILE,sizeof(char));
+  io->out_ancestral_tree_file           = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_boot_tree_file                = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_boot_stats_file               = (char *)mCalloc(T_MAX_FILE,sizeof(char));
   io->out_stats_file                    = (char *)mCalloc(T_MAX_FILE,sizeof(char));
@@ -1065,11 +1111,25 @@ eigen *Make_Eigen_Struct(int ns)
   eig->space_int   = (int *)mCalloc(2*ns,sizeof(int));
   eig->e_val       = (phydbl *)mCalloc(ns,sizeof(phydbl));
   eig->e_val_im    = (phydbl *)mCalloc(ns,sizeof(phydbl));
-  eig->r_e_vect    = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
   eig->r_e_vect_im = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
-  eig->l_e_vect    = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
   eig->q           = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
 
+#if (defined(__AVX__) || defined(__SSE3__))
+#ifndef WIN32
+  if(posix_memalign((void **)&eig->r_e_vect,BYTE_ALIGN,ns*ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&eig->l_e_vect,BYTE_ALIGN,ns*ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+  if(posix_memalign((void **)&eig->dum,BYTE_ALIGN,ns*ns*sizeof(phydbl))) Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
+#else
+  eig->r_e_vect = _aligned_malloc(ns*ns*sizeof(phydbl),BYTE_ALIGN);
+  eig->l_e_vect = _aligned_malloc(ns*ns*sizeof(phydbl),BYTE_ALIGN);
+  eig->dum      = _aligned_malloc(ns*ns*sizeof(phydbl),BYTE_ALIGN);
+#endif
+#else
+  eig->r_e_vect = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
+  eig->l_e_vect = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
+  eig->dum      = (phydbl *)mCalloc(ns*ns,sizeof(phydbl));
+#endif
+  
   Init_Eigen_Struct(eig);
 
   return eig;
@@ -1078,54 +1138,6 @@ eigen *Make_Eigen_Struct(int ns)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-triplet *Make_Triplet_Struct(t_mod *mod)
-{
-  int i,j,k;
-  triplet *triplet_struct;
-
-  triplet_struct                  = (triplet *)mCalloc(1,sizeof(triplet));
-  triplet_struct->size            = mod->ns;
-  triplet_struct->pi_bc           = (phydbl *)mCalloc(mod->ns,sizeof(phydbl ));
-  triplet_struct->pi_cd           = (phydbl *)mCalloc(mod->ns,sizeof(phydbl ));
-  triplet_struct->pi_bd           = (phydbl *)mCalloc(mod->ns,sizeof(phydbl ));
-  triplet_struct->F_bc            = (phydbl *)mCalloc(mod->ns*mod->ns*mod->ras->n_catg,sizeof(phydbl));
-  triplet_struct->F_cd            = (phydbl *)mCalloc(mod->ns*mod->ns*mod->ras->n_catg,sizeof(phydbl));
-  triplet_struct->F_bd            = (phydbl *)mCalloc(mod->ns*mod->ns,sizeof(phydbl));
-  triplet_struct->core            = (phydbl ****)mCalloc(mod->ras->n_catg,sizeof(phydbl ***));
-  triplet_struct->p_one_site      = (phydbl ***)mCalloc(mod->ns,sizeof(phydbl **));
-  triplet_struct->sum_p_one_site  = (phydbl ***)mCalloc(mod->ns,sizeof(phydbl **));
-  triplet_struct->eigen_struct    = (eigen *)Make_Eigen_Struct(mod->ns);
-  triplet_struct->mod             = mod;
-
-  for(k=0;k<mod->ras->n_catg;k++)
-    {
-      triplet_struct->core[k]                = (phydbl ***)mCalloc(mod->ns,sizeof(phydbl **));
-      for(i=0;i<mod->ns;i++)
-    {
-      triplet_struct->core[k][i]         = (phydbl **)mCalloc(mod->ns,sizeof(phydbl *));
-      for(j=0;j<mod->ns;j++)
-        triplet_struct->core[k][i][j]    = (phydbl  *)mCalloc(mod->ns,sizeof(phydbl ));
-    }
-    }
-
-  for(i=0;i<mod->ns;i++)
-    {
-      triplet_struct->p_one_site[i]          = (phydbl **)mCalloc(mod->ns,sizeof(phydbl *));
-      for(j=0;j<mod->ns;j++)
-    triplet_struct->p_one_site[i][j]     = (phydbl  *)mCalloc(mod->ns,sizeof(phydbl ));
-    }
-
-  for(i=0;i<mod->ns;i++)
-    {
-      triplet_struct->sum_p_one_site[i]      = (phydbl **)mCalloc(mod->ns,sizeof(phydbl *));
-      for(j=0;j<mod->ns;j++)
-    triplet_struct->sum_p_one_site[i][j] = (phydbl  *)mCalloc(mod->ns,sizeof(phydbl ));
-    }
-
-  return triplet_struct;
-
-}
-
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
@@ -1197,6 +1209,16 @@ void Make_Best_Spr(t_tree *tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+void Make_Spr(t_tree *tree)
+{
+  Make_Spr_List_One_Edge(tree);
+  Make_Spr_List_All_Edge(tree);
+  Make_Best_Spr(tree);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 void Make_Spr_List_One_Edge(t_tree *tree)
 {
   int i;
@@ -1282,8 +1304,8 @@ t_rate *RATES_Make_Rate_Struct(int n_otu)
       rates->mean_l               = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
       rates->ml_l                 = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
       rates->cur_l                = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
-      rates->u_ml_l               = (phydbl *)mCalloc(2*n_otu-3,sizeof(phydbl));
-      rates->u_cur_l              = (phydbl *)mCalloc(2*n_otu-3,sizeof(phydbl));
+      rates->u_ml_l               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      rates->u_cur_l              = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->cov_r                = (phydbl *)mCalloc((2*n_otu-2)*(2*n_otu-2),sizeof(phydbl));
       rates->cond_var             = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
       rates->mean_r               = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
@@ -1314,7 +1336,7 @@ t_rate *RATES_Make_Rate_Struct(int n_otu)
       rates->t_prior_max_ori      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->times_partial_proba  = (phydbl *)mCalloc(n_otu*n_otu,sizeof(phydbl));
       rates->numb_calib_chosen    = (int *)mCalloc(n_otu*n_otu,sizeof(phydbl));
-      rates->a_cal                = (t_cal **)mCalloc(n_otu-1,sizeof(t_cal *));
+      rates->a_cal                = (t_cal **)mCalloc(n_otu,sizeof(t_cal *));
       rates->model_name           = (char *)mCalloc(T_MAX_NAME,sizeof(char));
     }
 
@@ -1639,14 +1661,14 @@ t_ldsk *PHYREX_Make_Lindisk_Node(int n_dim)
 
 void PHYREX_Make_Lindisk_Next(t_ldsk *t)
 {
+  
   if(t->n_next == 0)
     t->next = (t_ldsk **)mCalloc(NEXT_BLOCK_SIZE,sizeof(t_ldsk *));
   else if(!(t->n_next%NEXT_BLOCK_SIZE))
     t->next = (t_ldsk **)mRealloc(t->next,t->n_next+NEXT_BLOCK_SIZE,sizeof(t_ldsk *));
 
   t->n_next++;
-  /* printf("\n. make next for ldsk %s n_next set to %d",t->coord->id,t->n_next); */
-  /* fflush(NULL); */
+  /* PhyML_Printf("\n. make next for ldsk %s n_next set to %d [%p] %d",t->coord->id,t->n_next,t->next,NEXT_BLOCK_SIZE); */
 }
 
 //////////////////////////////////////////////////////////////
