@@ -15,34 +15,12 @@ the GNU public licence. See http://www.opensource.org for details.
 #ifndef MCMC_H
 #define MCMC_H
 
-#include "spr.h"
 #include "utilities.h"
-#include "lk.h"
-#include "optimiz.h"
-#include "bionj.h"
-#include "models.h"
-#include "free.h"
-#include "help.h"
-#include "simu.h"
-#include "eigen.h"
-#include "pars.h"
-#include "alrt.h"
-#include "times.h"
-#include "m4.h"
-#include "draw.h"
-#include "rates.h"
-#include "stats.h"
-#include "phyrex.h"
-#include <stdlib.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <signal.h>
 
 void MCMC_Lexp(t_tree *tree);
 void MCMC_Print_Param(t_mcmc *mcmc, t_tree *tree);
 t_mcmc *MCMC_Make_MCMC_Struct();
 void MCMC_Free_MCMC(t_mcmc *mcmc);
-void MCMC(t_tree *tree);
 void MCMC_Alpha(t_tree *tree);
 void MCMC_Randomize_Branch_Lengths(t_tree *tree);
 void MCMC_Randomize_Node_Times(t_tree *tree);
@@ -162,14 +140,13 @@ void MCMC_PHYREX_Scale_Times(t_tree *tree);
 void MCMC_PHYREX_Ldscape_Limits(t_tree *tree);
 void MCMC_PHYREX_Indel_Disk(t_tree *tree);
 void MCMC_PHYREX_Delete_Disk(phydbl hr, int n_delete_disks, phydbl cur_lbda, phydbl cur_mu, phydbl cur_rad, t_tree *tree);
-void MCMC_PHYREX_Insert_Disk(phydbl hr, int n_delete_disks, phydbl cur_lbda, phydbl cur_mu, phydbl cur_rad, t_tree *tree);
+void MCMC_PHYREX_Insert_Disk(phydbl hr, int n_insert_disks, phydbl cur_lbda, phydbl cur_mu, phydbl cur_rad, t_tree *tree);
 void MCMC_PHYREX_Indel_Hit(t_tree *tree);
-void MCMC_PHYREX_Insert_Hit(phydbl hr, int n_insert_disks, phydbl cur_rad, phydbl cur_mu, t_tree *tree);
+void MCMC_PHYREX_Insert_Hit(phydbl hr, int n_delete_disks, phydbl cur_rad, phydbl cur_mu, t_tree *tree);
 void MCMC_PHYREX_Delete_Hit(phydbl hr, int n_delete_disks, phydbl cur_rad, phydbl cur_mu, t_tree *tree);
 void MCMC_PHYREX_Simulate_Backward(t_tree *tree);
 void MCMC_Update_Mode(int move_num, t_mcmc *mcmc, t_tree *tree);
 void MCMC_PHYREX_Lineage_Traj(t_tree *tree);
-void MCMC_PHYREX_Lbda_Times(t_tree *tree);
 void MCMC_PHYREX_Delete_Disk_Serial(t_tree *tree);
 void MCMC_PHYREX_Insert_Disk_Serial(t_tree *tree);
 void MCMC_PHYREX_Indel_Disk_Serial(t_tree *tree);
@@ -180,6 +157,7 @@ void MCMC_PHYREX_Ldsk_Multi(t_tree *tree);
 void MCMC_PHYREX_Ldsk_Given_Disk(t_tree *tree);
 void MCMC_PHYREX_Disk_Given_Ldsk(t_tree *tree);
 void MCMC_PHYREX_Ldsk_And_Disk(t_tree *tree);
+void MCMC_PHYREX_Add_Remove_Jump(t_tree *tree);
 void MCMC_Death_Rate(t_tree *tree);
 void MCMC_Times_All(t_tree *tree);
 void MCMC_Prune_Regraft(t_tree *tree);
@@ -194,4 +172,5 @@ void MCMC_Times_And_Rates_Recur(t_node *a, t_node *d, int traversal, t_tree *tre
 void MCMC_Times_And_Rates_All(t_tree *tree);
 void MCMC_Times_And_Rates_Root(t_tree *tree);
 void MCMC_Clade_Change(t_tree *tree);
+void MCMC_PHYREX_Prune_Regraft_Local(t_tree *tree);
 #endif
