@@ -104,7 +104,6 @@ void PMat_TN93(phydbl l, t_mod *mod, int pos, phydbl *Pij)
   kappa2 = mod->kappa->v*2./(1.+mod->lambda->v);
   kappa1 = kappa2 * mod->lambda->v;
 
-
   bt = l/(2.*(A*G*kappa1+C*T*kappa2+R*Y));
 
   a1t = kappa1;
@@ -115,7 +114,6 @@ void PMat_TN93(phydbl l, t_mod *mod, int pos, phydbl *Pij)
   e2 = (phydbl)exp(-a2t*Y-bt*R);
   e3 = (phydbl)exp(-bt);
 
-
   /*A->A*/Pij[pos + 4*0+0] = A+Y*A/R*e3+G/R*e1;
   /*A->C*/Pij[pos + 4*0+1] = C*(1-e3);
   /*A->G*/Pij[pos + 4*0+2] = G+Y*G/R*e3-G/R*e1;
@@ -248,86 +246,61 @@ phydbl Get_Lambda_F84(phydbl *pi, phydbl *kappa)
 /*   8000 = 20x20x20 times the operation +                          */
 /********************************************************************/
 
-void PMat_Empirical(phydbl l, t_mod *mod, int pos, phydbl *Pij, phydbl *tPij)
+void PMat_Empirical(const phydbl l, const t_mod *mod, const int pos, phydbl *Pij, phydbl *tPij)
 {
   const unsigned int ns = mod->ns;
   unsigned int i, j, k;
-  phydbl *U,*V,*R;
+  const phydbl *U,*V,*R;
   phydbl *expt;
   phydbl *uexpt;
   phydbl sum;
 
+  assert(Pij);
+  
   expt  = mod->eigen->e_val_im;
   uexpt = mod->eigen->r_e_vect_im;
   U     = mod->eigen->r_e_vect;
   V     = mod->eigen->l_e_vect;
   R     = mod->eigen->e_val; /* exponential of the eigen value matrix */
 
-  /* Initialize a rate-specific N*N matrix */
-  for(i=0;i<ns;++i) for(k=0;k<ns;k++) Pij[pos+ns*i+k] = .0;
-  /* compute POW(exp(D/mr),l) into mat_eDmrl */
-  for(k=0;k<ns;++k)  expt[k] = (phydbl)pow(R[k],l);
+  for(k=0;k<ns;++k)  expt[k] = exp(R[k]*l);
 
   /* multiply Vr*POW(exp(D/mr),l)*Vi into Pij */
   for(i=0;i<ns;i++) for (k=0;k<ns;k++) uexpt[i*ns+k] = U[i*ns+k] * expt[k];
-
+  
+  Pij = Pij + pos;
+  if(tPij != NULL) tPij = tPij + pos;
+  
   for(i=0;i<ns;++i)
     {
       for(j=0;j<ns;++j)
         {
+          Pij[j] = 0.0;
           for(k=0;k<ns;++k)
             {
-              Pij[pos+ns*i+j] += (uexpt[i*ns+k] * V[k*ns+j]);
+              Pij[j] += (uexpt[i*ns+k] * V[k*ns+j]);
             }
-          if(Pij[pos+ns*i+j] < SMALL_PIJ) Pij[pos+ns*i+j] = SMALL_PIJ;
+          if(Pij[j] < SMALL_PIJ) Pij[j] = SMALL_PIJ;
         }
-
       sum = 0.0;
-      for(j=0;j<ns;++j) sum += Pij[pos+ns*i+j];
-      for(j=0;j<ns;++j) Pij[pos+ns*i+j] /= sum;
+      for(j=0;j<ns;++j) sum += Pij[j];
+      for(j=0;j<ns;++j) Pij[j] /= sum;
+
+      Pij += ns;
     }
+  
+  Pij -= ns*ns;
 
-
-  if(tPij)
+  if(tPij != NULL)
     {
       for(i=0;i<ns;++i)
         {
           for(j=0;j<ns;++j)
             {
-              tPij[pos+ns*i+j] = Pij[pos+ns*j+i];
+              tPij[ns*i+j] = Pij[ns*j+i];
             }
         }
     }
-
-  /* PhyML_Printf("\n. l: %G",l); */
-  /* PhyML_Printf("\n. U\n"); */
-  /* for(i=0;i<ns;++i) */
-  /*   { */
-  /*     for(j=0;j<ns;++j) */
-  /*       { */
-  /*         PhyML_Printf("%5f ",U[ns*i+j]); */
-  /*       } */
-  /*     PhyML_Printf("\n"); */
-  /*   } */
-  /* PhyML_Printf("\n. V\n"); */
-  /* for(i=0;i<ns;++i) */
-  /*   { */
-  /*     for(j=0;j<ns;++j) */
-  /*       { */
-  /*         PhyML_Printf("%5f ",V[i*ns+j]); */
-  /*       } */
-  /*     PhyML_Printf("\n"); */
-  /*   } */
-  /* PhyML_Printf("\n. P\n"); */
-  /* for(i=0;i<ns;++i) */
-  /*   { */
-  /*     for(j=0;j<ns;++j) */
-  /*       { */
-  /*         PhyML_Printf("%5f ",Pij[pos+ns*j+i]); */
-  /*       } */
-  /*     PhyML_Printf("\n"); */
-  /*   } */
-
 }
 
 //////////////////////////////////////////////////////////////
@@ -579,12 +552,12 @@ void Update_Qmat_Generic(phydbl *rr, phydbl *pi, int ns, phydbl *qmat)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-void Update_Qmat_GTR(phydbl *rr, phydbl *rr_val, int *rr_num, phydbl *pi, phydbl *qmat)
+void Update_Qmat_GTR(phydbl *rr, phydbl *rr_val, int *rr_num, phydbl *pi, phydbl *qmat, int opt_rr)
 {
   int i;
   phydbl mr;
 
-  for(i=0;i<6;i++) rr[i] = rr_val[rr_num[i]];
+  for(i=0;i<6;i++) rr[i] = exp(rr_val[rr_num[i]]);
   for(i=0;i<6;i++)
     if(rr[i] < 0.0)
       {
@@ -592,7 +565,8 @@ void Update_Qmat_GTR(phydbl *rr, phydbl *rr_val, int *rr_num, phydbl *pi, phydbl
         PhyML_Fprintf(stderr,"\n. Err. in file %s at line %d (function '%s').\n",__FILE__,__LINE__,__FUNCTION__);
         Exit("");
       }
-  for(i=0;i<6;i++) rr[i] /= MAX(rr[5],RR_MIN);
+
+  for(i=0;i<6;i++) rr[i] /= rr[5];
   for(i=0;i<6;i++) if(rr[i] < RR_MIN) rr[i] = RR_MIN;
   for(i=0;i<6;i++) if(rr[i] > RR_MAX) rr[i] = RR_MAX;
 
@@ -621,20 +595,6 @@ void Update_Qmat_GTR(phydbl *rr, phydbl *rr_val, int *rr_num, phydbl *pi, phydbl
   mr = .0;
   for(i=0;i<4;i++) mr += pi[i] * (-qmat[i*4+i]);
   for(i=0;i<16;i++) qmat[i] /= mr;
-
-  /* int j; */
-  /* printf("\n"); */
-  /* printf("\n. rr -- "); */
-  /* for(i=0;i<5;i++) printf(" %15f ",rr[i]); */
-  /* printf("\n"); */
-  /* for(i=0;i<4;i++) */
-  /*   { */
-  /*     printf("\n. [%15f] \t ",pi[i]); */
-  /*     for(j=0;j<4;j++) */
-  /* 	{ */
-  /* 	  printf(" %15f ",qmat[i*4+j]); */
-  /* 	} */
-  /*   } */
 }
 
 //////////////////////////////////////////////////////////////
@@ -802,14 +762,18 @@ void Update_Efrq(t_mod *mod)
 
   if((mod->io->datatype == NT) && (mod->s_opt->opt_state_freq == YES))
     {
+      for(i=0;i<mod->ns;++i) mod->e_frq->pi->v[i] = exp(mod->e_frq->pi_unscaled->v[i]);
       sum = 0.0;
-      for(i=0;i<mod->ns;i++) sum += fabs(mod->e_frq->pi_unscaled->v[i]);
-      for(i=0;i<mod->ns;i++) mod->e_frq->pi->v[i] = fabs(mod->e_frq->pi_unscaled->v[i])/sum;
-            
+      for(i=0;i<mod->ns;++i) sum += mod->e_frq->pi->v[i];
+      for(i=0;i<mod->ns;++i) mod->e_frq->pi->v[i] = mod->e_frq->pi->v[i]/sum;
+
 #ifdef BEAGLE
       if(UNINITIALIZED != mod->b_inst)
         update_beagle_efrqs(mod);
 #endif
+
+      for(i=0;i<mod->ns;++i) if(mod->e_frq->pi->v[i] < E_FRQ_MIN) mod->e_frq->pi->v[i] = E_FRQ_MIN;
+      for(i=0;i<mod->ns;++i) if(mod->e_frq->pi->v[i] > E_FRQ_MAX) mod->e_frq->pi->v[i] = E_FRQ_MAX;
     }
   
 }
@@ -831,7 +795,7 @@ void Set_Model_Parameters(t_mod *mod)
 void Update_Boundaries(t_mod *mod)
 {
   int i;
-
+    
   if(mod->kappa->v > TSTV_MAX) mod->kappa->v = TSTV_MAX;
   if(mod->kappa->v < TSTV_MIN) mod->kappa->v = TSTV_MIN;
 
@@ -858,8 +822,11 @@ void Update_Boundaries(t_mod *mod)
 
   if(mod->whichmodel == CUSTOM || mod->whichmodel == GTR)
     {
-      for(i=0;i<6;i++) if(mod->r_mat->rr_val->v[i] < RR_MIN) mod->r_mat->rr_val->v[i] = RR_MIN;
-      for(i=0;i<6;i++) if(mod->r_mat->rr_val->v[i] > RR_MAX) mod->r_mat->rr_val->v[i] = RR_MAX;
+      for(i=0;i<6;i++) if(mod->r_mat->rr_val->v[i] < UNSCALED_RR_MIN) mod->r_mat->rr_val->v[i] = UNSCALED_RR_MIN;
+      for(i=0;i<6;i++) if(mod->r_mat->rr_val->v[i] > UNSCALED_RR_MAX) mod->r_mat->rr_val->v[i] = UNSCALED_RR_MAX;
+
+      for(i=0;i<6;i++) if(mod->r_mat->rr->v[i] < RR_MIN) mod->r_mat->rr->v[i] = RR_MIN;
+      for(i=0;i<6;i++) if(mod->r_mat->rr->v[i] > RR_MAX) mod->r_mat->rr->v[i] = RR_MAX;
     }
 
   for(i=0;i<mod->ns;i++)
@@ -869,6 +836,12 @@ void Update_Boundaries(t_mod *mod)
 
       if(mod->e_frq->pi_unscaled->v[i] > UNSCALED_E_FRQ_MAX)
         mod->e_frq->pi_unscaled->v[i] = UNSCALED_E_FRQ_MAX;
+            
+      if(mod->e_frq->pi->v[i] < E_FRQ_MIN)
+        mod->e_frq->pi->v[i] = E_FRQ_MIN;
+
+      if(mod->e_frq->pi->v[i] > E_FRQ_MAX)
+        mod->e_frq->pi->v[i] = E_FRQ_MAX;
     }
   
   if(mod->r_mat_weight->v < R_MAT_WEIGHT_MIN) mod->r_mat_weight->v = R_MAT_WEIGHT_MIN;
@@ -887,15 +860,7 @@ void Update_Eigen(t_mod *mod)
   int result, n_iter;
   phydbl scalar;
   int i;
-
-
-  if(mod->is_mixt_mod == YES)
-    {
-      MIXT_Update_Eigen(mod);
-      return;
-    }
-
-
+  
   if(mod->update_eigen == YES)
     {
       //Update the Q-matrix first before computing the Eigen(because the Eigen is computed based on the Q-matrix)
@@ -904,9 +869,9 @@ void Update_Eigen(t_mod *mod)
           if(mod->io->datatype == NT)
             {
               if(mod->whichmodel == GTR)
-                Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v);
+                Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v, mod->s_opt->opt_rr);
               else if(mod->whichmodel == CUSTOM)
-                Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v);
+                Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v, mod->s_opt->opt_rr);
               else if(mod->whichmodel == HKY85)
                 Update_Qmat_HKY(mod->kappa->v, mod->e_frq->pi->v, mod->r_mat->qmat->v);
               else /* Any other nucleotide-based t_mod */
@@ -927,31 +892,41 @@ void Update_Eigen(t_mod *mod)
       n_iter   = 0;
       result   = 0;
 
+      
       for(i=0;i<mod->ns*mod->ns;++i)
         {
           mod->r_mat->qmat_buff->v[i] = mod->r_mat->qmat->v[i];
         }
       
+      /* For(i,mod->eigen->size*mod->eigen->size) PhyML_Printf("\n. qmat [%d,%d]=%f",i/mod->eigen->size,i%mod->eigen->size,mod->r_mat->qmat->v[i]); */
+      /* for(i=0;i<6;i++) PhyML_Printf("\n. rr: %f rr_Val: %f",mod->r_mat->rr_val->v[i],mod->r_mat->rr->v[i]); */
+      /* for(i=0;i<4;i++) PhyML_Printf("\n. f: %f %f",mod->e_frq->pi->v[i],mod->e_frq->pi_unscaled->v[i]); */
+      /* PhyML_Printf("\n"); */
+      
       /* compute eigenvectors/values */
       /*       if(!EigenRealGeneral(mod->eigen->size,mod->r_mat->qmat,mod->eigen->e_val, */
       /* 			  mod->eigen->e_val_im, mod->eigen->r_e_vect, */
       /* 			  mod->eigen->space_int,mod->eigen->space)) */
 
       if(!Eigen(1,mod->r_mat->qmat_buff->v,mod->eigen->size,mod->eigen->e_val,
-                mod->eigen->e_val_im,mod->eigen->r_e_vect,
-                mod->eigen->r_e_vect_im,mod->eigen->space))
+                mod->eigen->e_val_im,
+                mod->eigen->r_e_vect,
+                mod->eigen->r_e_vect_im,
+                mod->eigen->space))
         {
           /* compute inverse(Vr) into Vi */
-          For (i,mod->ns*mod->ns) mod->eigen->l_e_vect[i] = mod->eigen->r_e_vect[i];
+          for (i=0;i<mod->ns*mod->ns;++i) mod->eigen->l_e_vect[i] = mod->eigen->r_e_vect[i];
           while(!Matinv(mod->eigen->l_e_vect, mod->eigen->size, mod->eigen->size,YES))
             {
 	      PhyML_Printf("\n. Trying Q<-Q*scalar and then Root<-Root/scalar to fix this...\n");
               scalar += scalar / 3.;
+              For(i,mod->eigen->size*mod->eigen->size) PhyML_Printf("\n. qmat [%d,%d]=%f",i/mod->eigen->size,i%mod->eigen->size,mod->r_mat->qmat->v[i]);
+
               For(i,mod->eigen->size*mod->eigen->size) mod->r_mat->qmat_buff->v[i]  = mod->r_mat->qmat->v[i];
               For(i,mod->eigen->size*mod->eigen->size) mod->r_mat->qmat_buff->v[i] *= scalar;
               result = Eigen(1,mod->r_mat->qmat_buff->v,mod->eigen->size,mod->eigen->e_val,
-                     mod->eigen->e_val_im,mod->eigen->r_e_vect,
-                     mod->eigen->r_e_vect_im,mod->eigen->space);
+                             mod->eigen->e_val_im,mod->eigen->r_e_vect,
+                             mod->eigen->r_e_vect_im,mod->eigen->space);
               if (result == -1)
                 {
                   PhyML_Fprintf(stderr,"\n. Eigenvalues/vectors computation did not converge: computation cancelled."); 
@@ -973,41 +948,11 @@ void Update_Eigen(t_mod *mod)
             }
           for(i=0;i<mod->eigen->size;i++) mod->eigen->e_val[i] /= scalar;
           
-          /* compute the diagonal terms of exp(D) */
-          for(i=0;i<mod->ns;i++) mod->eigen->e_val[i] = (phydbl)exp(mod->eigen->e_val[i]);
-
-      /* int j; */
-      /* double *U,*V,*R; */
-      /* double *expt; */
-      /* double *uexpt; */
-      /* int n; */
-
-      /* expt  = mod->eigen->e_val_im; */
-      /* uexpt = mod->eigen->r_e_vect_im; */
-      /* U     = mod->eigen->r_e_vect; */
-      /* V     = mod->eigen->l_e_vect; */
-      /* R     = mod->eigen->e_val; /\* exponential of the eigen value matrix *\/ */
-      /* n     = mod->ns; */
-
-      /* PhyML_Printf("\n"); */
-      /* PhyML_Printf("\n. Q\n"); */
-      /* for(i=0;i<n;i++) { for(j=0;j<n;j++) PhyML_Printf("%7.3f ",mod->eigen->q[i*n+j]); PhyML_Printf("\n"); } */
-      /* PhyML_Printf("\n. U\n"); */
-      /* for(i=0;i<n;i++) { for(j=0;j<n;j++) PhyML_Printf("%7.3f ",U[i*n+j]); PhyML_Printf("\n"); } */
-      /* PhyML_Printf("\n"); */
-      /* PhyML_Printf("\n. V\n"); */
-      /* for(i=0;i<n;i++) { for(j=0;j<n;j++) PhyML_Printf("%7.3f ",V[i*n+j]); PhyML_Printf("\n"); } */
-      /* PhyML_Printf("\n"); */
-      /* PhyML_Printf("\n. Eigen\n"); */
-      /* for(i=0;i<n;i++)  PhyML_Printf("%E ",mod->eigen->e_val[i]); */
-      /* PhyML_Printf("\n"); */
-
-/* 	  Exit("\n"); */
 #ifdef BEAGLE
           //Recall that BEAGLE is initialized *after* all the model parameters are set
           //IOW, this function may be called before BEAGLE is initialized ("chicken-egg")
           if(UNINITIALIZED != mod->b_inst)
-              update_beagle_eigen(mod);
+            update_beagle_eigen(mod);
 #endif
         }
       else
@@ -1016,7 +961,6 @@ void Update_Eigen(t_mod *mod)
           Warn_And_Exit("\n");
         }
     }
-
 }
 
 //////////////////////////////////////////////////////////////
@@ -1030,7 +974,7 @@ void Switch_From_M4mod_To_Mod(t_mod *mod)
   mod->ns = mod->m4mod->n_o;
   for(i=0;i<mod->ns;i++) mod->e_frq->pi->v[i] = mod->m4mod->o_fq[i];
   mod->eigen->size = mod->ns;
-  Switch_Eigen(YES,mod);
+  Set_Update_Eigen(YES,mod);
 }
 
 //////////////////////////////////////////////////////////////
@@ -1097,13 +1041,12 @@ void Switch_From_Mod_To_M4mod(t_mod *mod)
   mod->ns = mod->m4mod->n_o * mod->m4mod->n_h;
   for(i=0;i<mod->ns;i++) mod->e_frq->pi->v[i] = mod->m4mod->o_fq[i%mod->m4mod->n_o] * mod->m4mod->h_fq[i/mod->m4mod->n_o];
   mod->eigen->size = mod->ns;
-  Switch_Eigen(YES,mod);
+  Set_Update_Eigen(YES,mod);
 }
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 phydbl General_Dist(phydbl *F, t_mod *mod, eigen *eigen_struct)
 {
   phydbl *pi,*mod_pi;
@@ -1143,7 +1086,7 @@ phydbl General_Dist(phydbl *F, t_mod *mod, eigen *eigen_struct)
        mod->eigen->r_e_vect_im,mod->eigen->space))
     {
       for(i=0;i<mod->ns;i++) mod->e_frq->pi->v[i] = mod_pi[i];
-      Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v);
+      Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v, mod->s_opt->opt_rr);
       Free(pi);
       Free(mod_pi);
       return -1.;
@@ -1154,7 +1097,7 @@ phydbl General_Dist(phydbl *F, t_mod *mod, eigen *eigen_struct)
   if(!Matinv(eigen_struct->l_e_vect,eigen_struct->size,eigen_struct->size,YES))
     {
       for(i=0;i<mod->ns;i++) mod->e_frq->pi->v[i] = mod_pi[i];
-      Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v);
+      Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v, mod->s_opt->opt_rr);
       Free(pi);
       Free(mod_pi);
       return -1.;
@@ -1187,9 +1130,6 @@ phydbl General_Dist(phydbl *F, t_mod *mod, eigen *eigen_struct)
 /*   dist /= sum_ev; */
   dist /= -4.;
 
-
-/*   for(i=0;i<mod->ns;i++) mod->e_frq->pi->v[i] = mod_pi[i]; */
-/*   Update_Qmat_GTR(mod); */
   Free(pi);
   Free(mod_pi);
   Free(F_phydbl);
@@ -1281,7 +1221,6 @@ phydbl GTR_Dist(phydbl *F, phydbl alpha, eigen *eigen_struct)
   dist = .0;
   for(i=0;i<eigen_struct->size;i++) dist-=F[eigen_struct->size*i+i];
 
-/*   PhyML_Printf("\nDIST = %f\n",dist); Exit("\n"); */
 
   Free(pi);
   Free(F_phydbl);
