@@ -20,7 +20,7 @@ the GNU public licence. See http://www.opensource.org for details.
    if tbe_bootstrap == 0 => Classical FBP (Felsenstein bootstrap proportions) 
    else => TBE (Transfer bootstrap expectation)
 */
-void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
+void Bootstrap_MPI(t_tree *tree)
 {
   int *site_num, n_site;
   int replicate,j,k;
@@ -45,11 +45,9 @@ void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
 
   randomRecv = nbElem = bootRecv = 0;
 
-  tree->print_boot_val       = !tbe_bootstrap;
-  tree->print_tbe_val        = tbe_bootstrap;
+  tree->io->print_support_val = YES;
   
-  tree->print_alrt_val       = 0;
-  boot_tree                  = NULL;
+  boot_tree = NULL;
 
   site_num = (int *)mCalloc(tree->data->init_len,sizeof(int));
   
@@ -66,28 +64,25 @@ void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
 
   boot_data = Copy_Cseq(tree->data,tree->io);
 
-  if (Global_myRank == 0)
-    PhyML_Printf("\n. Non parametric bootstrap analysis \n");
-
   if(Global_numTask <= 1)
     {
-      PhyML_Printf("\n. The number of CPUs used in the MPI bootstrap analysis should be");
-      PhyML_Printf("\n. strictly greater than 1.");
+      PhyML_Printf("\n\n. The number of CPUs used in the MPI bootstrap analysis should be");
+      PhyML_Printf("\n. strictly greater than 1 (it is equal to %d here).",Global_numTask);
       assert(FALSE);
     }
   
   //number of bootstraps for each process
-  if (tree->mod->bootstrap%Global_numTask != 0) 
+  if (tree->io->n_boot_replicates%Global_numTask != 0) 
     {
-      nbRep = (tree->mod->bootstrap / Global_numTask) + 1;
-      tree->mod->bootstrap = nbRep * Global_numTask;
+      nbRep = (tree->io->n_boot_replicates / Global_numTask) + 1;
+      tree->io->n_boot_replicates = nbRep * Global_numTask;
       if (Global_myRank == 0) {
-        PhyML_Printf("\n. The number of replicates is not a multiple of %d CPUs.\n", Global_numTask);
-        PhyML_Printf("\n. Will run %d replicates.\n", tree->mod->bootstrap);
+        PhyML_Printf("\n\n. The number of replicates is not a multiple of %d CPUs.\n", Global_numTask);
+        PhyML_Printf("\n. Will run %d replicates.\n", tree->io->n_boot_replicates);
       }
     }
   else
-    nbRep = tree->mod->bootstrap/Global_numTask;
+    nbRep = tree->io->n_boot_replicates/Global_numTask;
   
   //Bip score
   if (Global_myRank == 0) 
@@ -106,7 +101,7 @@ void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
   if (Global_myRank == 0)
     PhyML_Printf("\n  [");
 
-  For(replicate, nbRep)
+  for(replicate=0;replicate<nbRep;++replicate)
     {
       for(j=0;j<boot_data->crunch_len;j++) boot_data->wght[j] = 0;
 
@@ -114,10 +109,10 @@ void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
       if (Global_myRank == 0) 
         {
           // Compute number of data to send
-          if (tree->mod->bootstrap - randomRecv > Global_numTask)
+          if (tree->io->n_boot_replicates - randomRecv > Global_numTask)
             nbElem = Global_numTask;
           else
-            nbElem = tree->mod->bootstrap - randomRecv;
+            nbElem = tree->io->n_boot_replicates - randomRecv;
           
           for(i=0;i<nbElem;i++) 
             {
@@ -247,15 +242,14 @@ void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
       Fill_Dir_Table(boot_tree);
       Update_Dirs(boot_tree);
 
-      if(tree->mod->s_opt->greedy) Init_Partial_Lk_Tips_Double(boot_tree);
-      else                         Init_Partial_Lk_Tips_Int(boot_tree);
+      Init_Partial_Lk_Tips_Double(boot_tree);
       Init_Ui_Tips(boot_tree);
       Init_Partial_Pars_Tips(boot_tree);
       Br_Len_Not_Involving_Invar(boot_tree);
       
-      Switch_Eigen(YES,boot_tree->mod);                 
+      Set_Update_Eigen(YES,boot_tree->mod);                 
       Lk(NULL,boot_tree);
-      Switch_Eigen(NO,boot_tree->mod);
+      Set_Update_Eigen(NO,boot_tree->mod);
 
       if(boot_tree->mod->s_opt->opt_topo)
         {
@@ -279,16 +273,22 @@ void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
               boot_tree->a_nodes[0]->v[0],
               boot_tree);
 
-      if(!tbe_bootstrap){
-	Compare_Bip(tree,boot_tree,NO);
-      }else{
-	Compare_Bip_Distance(tree, boot_tree);
-      }
+      if(tree->io->do_boot)
+        {
+          Compare_Bip(tree,boot_tree,NO);
+        }
+      else if(tree->io->do_tbe)
+        {
+          Compare_Bip_Distance(tree, boot_tree);
+        }
+      else assert(FALSE);
+        
       
       Br_Len_Involving_Invar(boot_tree);
 
       if(tree->io->print_boot_trees)
         {
+          Insert_Duplicates(boot_tree);
           s = Write_Tree(boot_tree,NO);
           t=(char *)mCalloc(T_MAX_LINE,sizeof(char));
           Print_Fp_Out_Lines_MPI(boot_tree, tree->io, replicate+1, t);
@@ -301,15 +301,15 @@ void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
             PhyML_Printf(".");
 	    if(!((bootRecv)%tree->io->boot_prog_every)) 
 	      {
-		PhyML_Printf("] %4d/%4d\n  ",bootRecv,tree->mod->bootstrap);
-		if(bootRecv != tree->mod->bootstrap) PhyML_Printf("[");
+		PhyML_Printf("] %4d/%4d\n  ",bootRecv,tree->io->n_boot_replicates);
+		if(bootRecv != tree->io->n_boot_replicates) PhyML_Printf("[");
 	      }
 	    
             // Compute number of bootstraps to receive
-            if (tree->mod->bootstrap - bootRecv > Global_numTask)
+            if (tree->io->n_boot_replicates - bootRecv > Global_numTask)
               nbElem = Global_numTask;
             else
-              nbElem = tree->mod->bootstrap - bootRecv + 1;
+              nbElem = tree->io->n_boot_replicates - bootRecv + 1;
               
             bootStr=(char *)mCalloc(T_MAX_LINE,sizeof(char));
             for (i=1; i<nbElem; i++) 
@@ -337,8 +337,8 @@ void Bootstrap_MPI(t_tree *tree, int tbe_bootstrap)
 		bootRecv++;
 		PhyML_Printf(".");
 		if(!((bootRecv)%tree->io->boot_prog_every)) {
-		  PhyML_Printf("] %4d/%4d\n  ",bootRecv,tree->mod->bootstrap);
-		  if(bootRecv != tree->mod->bootstrap) PhyML_Printf("[");
+		  PhyML_Printf("] %4d/%4d\n  ",bootRecv,tree->io->n_boot_replicates);
+		  if(bootRecv != tree->io->n_boot_replicates) PhyML_Printf("[");
 		}
 	      }
             Free(bootStr);
@@ -378,19 +378,19 @@ fflush(stdout);
       Free (score_tot);
     }
   Free (score_par);
-
+  
   if (Global_myRank == 0)
-    if(((bootRecv)%tree->io->boot_prog_every)) PhyML_Printf("] %4d/%4d\n ",bootRecv,tree->mod->bootstrap);
-
-PhyML_Printf("\n\n. Exiting bootstrap function normally."); fflush(NULL);
-tree->lock_topo = 1; /* Topology should not be modified afterwards */
-
+    if(((bootRecv)%tree->io->boot_prog_every)) PhyML_Printf("] %4d/%4d\n ",bootRecv,tree->io->n_boot_replicates);
+  
+  PhyML_Printf("\n\n. Exiting bootstrap function normally."); fflush(NULL);
+  tree->lock_topo = 1; /* Topology should not be modified afterwards */
+  
   if(tree->io->print_boot_trees)
     {
       fclose(tree->io->fp_out_boot_tree);
       fclose(tree->io->fp_out_boot_stats);
     }
-
+  
   Free_Calign(boot_data);
   Free(site_num);
 }
