@@ -17,6 +17,7 @@ the GNU public licence. See http://www.opensource.org for details.
 
 #include "utilities.h"
 
+void PHYREX_XML(char *xml_filename);
 int PHYREX_Main(int argc, char **argv);
 int PHYREX_Main_Simulate(int argc, char **argv);
 int PHYREX_Main_Estimate(int argc, char **argv);
@@ -38,7 +39,7 @@ void PHYREX_Update_Lindisk_List_Pre(t_dsk *disk, t_tree *tree);
 void PHYREX_Connect_Ldsk_Given_Disk(t_dsk **disk, int n_disk, t_ldsk *y_ldsk, t_ldsk *o_ldsk, int dir_o_y);
 void PHYREX_Print_Struct(char sign, t_tree *tree);
 phydbl PHYREX_Uniform_Path_Density(t_ldsk *y_ldsk, t_ldsk *o_ldsk, t_tree *tree);
-void PHYREX_Check_Struct(t_tree *tree);
+int PHYREX_Check_Struct(t_tree *tree);
 void PHYREX_Store_Geo_Coord(t_geo_coord *t);
 void PHYREX_Restore_Geo_Coord(t_geo_coord *t);
 int PHYREX_Total_Number_Of_Intervals(t_tree *tree);
@@ -66,7 +67,7 @@ void PHYREX_Ldsk_To_Tree(t_tree *tree);
 void PHYREX_Ldsk_To_Tree_Post(t_node *a, t_ldsk *ldsk, int *available, t_tree *tree);
 phydbl PHYREX_Rnorm_Trunc(t_ldsk *ldsk, t_dsk *disk, t_phyrex_mod *mod);
 void PHYREX_Remove_Lindisk_Next(t_ldsk *ldsk, t_ldsk *rm);
-phydbl PHYREX_Simulate_Backward_Core(int new_loc, t_dsk *init_disk, t_tree *tree);
+phydbl PHYREX_Simulate_Backward_Core(t_dsk *init_disk, int avoid_multiple_mergers, t_tree *tree);
 phydbl *PHYREX_Mean_Pairwise_Distance_Between_Lineage_Locations(t_tree *tree);
 phydbl PHYREX_Random_Select_Time_Between_Jumps(t_tree *tree);
 t_sarea *PHYREX_Simulate_Forward_Core(int n_sites, t_tree *tree);
@@ -79,7 +80,7 @@ phydbl PHYREX_Neighborhood_Size_Regression(t_tree *tree);
 phydbl PHYREX_Neighborhood_Size(t_tree *tree);
 phydbl PHYREX_Update_Radius(t_tree *tree);
 phydbl PHYREX_Update_Sigsq(t_tree *tree);
-void PHYREX_Read_Tip_Coordinates(t_ldsk **ldsk_a, t_tree *tree);
+void PHYREX_Read_Tip_Coordinates(t_tree *tree);
 phydbl PHYREX_Sample_Rad_From_Prior(t_tree *tree);
 void MCMC_PHYREX_Sigsq(t_tree *tree);
 phydbl PHYREX_LnPrior_Sigsq(t_tree *tree);
@@ -90,7 +91,7 @@ void PHYREX_Insert_Ldsk_In_Next_List(t_ldsk *ins, int pos, t_ldsk *where);
 t_ldsk *PHYREX_Remove_Path(t_ldsk *beg, t_ldsk *end, int *pos_end, t_tree *tree);
 void PHYREX_Insert_Path(t_ldsk *beg, t_ldsk *end, t_ldsk *path, int pos, t_tree *tree);
 t_ldsk *PHYREX_Generate_Path(t_ldsk *beg, t_ldsk *end, phydbl n_evt, phydbl sd, t_tree *tree);
-phydbl PHYREX_Path_Logdensity(t_ldsk *beg, t_ldsk *end, phydbl cur_n_evt, phydbl sd, t_tree *tree);
+phydbl PHYREX_Path_Logdensity(t_ldsk *beg, t_ldsk *end, phydbl sd, t_tree *tree);
 phydbl PHYREX_Time_Tree_Length(t_tree *tree);
 void PHYREX_Time_Tree_Length_Pre(t_ldsk *a, t_ldsk *d, phydbl *len, t_tree *tree);
 int PHYREX_Is_On_Path(t_ldsk *target, t_ldsk *beg, t_ldsk *end);
@@ -110,8 +111,17 @@ int PHYREX_Number_Of_Sampled_Demes(t_tree *tree);
 phydbl PHYREX_Coalescence_Rate(t_tree *tree);
 t_tree *PHYREX_Simulate_Independent_Loci(int n_otu, int n_loci, phydbl w, phydbl h, int r_seed);
 phydbl Prob_Two_Lineages_Coal_One_Event(phydbl w, phydbl h, phydbl mu, phydbl rad);
-
-
-
+void PHYREX_Tree_To_Ldsk(t_tree *tree);
+void PHYREX_Tree_To_Ldsk_Post(t_node *a, t_node *d, t_dsk *a_disk, t_tree *tree);
+void PHYREX_Make_And_Connect_Tip_Disks(t_tree *tree);
+void PHYREX_Simulate_Disk_And_Node_Times(t_tree *tree);
+int PHYREX_Number_Of_Outgoing_Ldsks(t_dsk *disk);
+t_ldsk *PHYREX_Random_Select_Outgoing_Ldsk(t_dsk *disk);
+void PHYREX_Strip_And_Reconnect_Tree(t_tree *tree);
+int PHYREX_Scale_All(phydbl scale, t_dsk *start_disk, t_tree *tree);
+void PHYREX_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr, t_tree *tree);
+void  PHYREX_Update_Lindisk_List_Range(t_dsk *young, t_dsk *old, t_tree *tree);
+phydbl PHYREX_Lk_Core_Range(t_dsk *young, t_dsk *old, t_tree *tree);
+int PHYREX_Number_Of_Intervals_Range(t_dsk *young, t_dsk *old, t_tree *tree);
 
 #endif
