@@ -139,8 +139,11 @@ int Spr(phydbl init_lnL, phydbl prop_spr, t_tree *tree)
 
       b = tree->a_edges[br];
       
-      Spr_Subtree(b,b->left,tree);
-      Spr_Subtree(b,b->rght,tree);
+      if(b->l->v > tree->mod->s_opt->l_min_spr)
+        {
+          Spr_Subtree(b,b->left,tree);
+          Spr_Subtree(b,b->rght,tree);
+        }
     }
 
   
@@ -732,12 +735,14 @@ void Global_Spr_Search(t_tree *tree)
   phydbl best_lnL;
   t_tree *best_tree;
   time_t t_cur;
-  phydbl mean_delta_lnL_spr,max_delta_lnL_spr;
+  phydbl mean_delta_lnL_spr,max_delta_lnL_spr,tune_l_mult;
   
   unsigned int no_improv  = 0;
   unsigned int last_best_found = 0;
   unsigned int hit_zero_improv = 0;
-
+  unsigned int freq = 1;
+  const unsigned int round_freq = 10;
+  
   best_lnL      = UNLIKELY;
   tree->verbose = (tree->verbose == VL0) ? VL0 : VL1;
   best_tree     = Make_Tree_From_Scratch(tree->n_otu,tree->data);
@@ -746,39 +751,23 @@ void Global_Spr_Search(t_tree *tree)
 
   if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf("\n\n. Score of initial tree: %.2f",tree->c_lnL);
 
+
   tree->mod->s_opt->min_diff_lk_move  = 1.E-1;
   tree->mod->s_opt->min_diff_lk_local = 1.E-1;
   Round_Optimize(tree,1000);
   best_lnL = tree->c_lnL;
   Copy_Tree(tree,best_tree);
 
-  if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf("\n\n. Score of first optimized tree: %.2f",tree->c_lnL);
-
-  if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf("\n\n. First round of optimization...\n");
-
-  iter = 0;
   
-  /* Stepwise_Add_Pars(tree); */
-  /* Spr_Pars(0,tree->n_otu,tree); */
-  /* Add_BioNJ_Branch_Lengths(tree,tree->data,tree->mod,NULL); */
-  
-  tree->mod->s_opt->spr_lnL           = YES;
-  tree->mod->s_opt->spr_pars          = NO;
-  tree->mod->s_opt->min_diff_lk_move  = 1.E-0;
-  tree->mod->s_opt->min_diff_lk_local = 1.E-0;
-  tree->mod->s_opt->eval_list_regraft = NO;
-  tree->mod->s_opt->min_n_triple_moves= 10;
-  tree->mod->s_opt->max_depth_path    = 50;
-  tree->mod->s_opt->max_delta_lnL_spr = 10000.;
-  Spr(tree->c_lnL,1.0,tree);
-  best_lnL = tree->c_lnL;;
-      
-  tree->mod->s_opt->max_depth_path            = MIN(20,(int)(tree->n_otu/4+1));
-  tree->mod->s_opt->max_delta_lnL_spr         = 1000.;
+  if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf("\n\n. Starting first round of SPRs...\n");
+
+  tree->mod->s_opt->max_depth_path            = MAX(40,1+(int)(tree->n_otu/4));
+  tree->mod->s_opt->max_delta_lnL_spr         = 1000;
+  tree->mod->s_opt->l_min_spr                 = 0.0;
   tree->mod->s_opt->spr_lnL                   = YES;
   tree->mod->s_opt->spr_pars                  = NO;
-  tree->mod->s_opt->min_diff_lk_move          = 1.E-1;
-  tree->mod->s_opt->min_diff_lk_local         = 1.E-1;
+  tree->mod->s_opt->min_diff_lk_move          = 1.E-0;
+  tree->mod->s_opt->min_diff_lk_local         = 1.E-0;
   tree->perform_spr_right_away                = YES;
   tree->mod->s_opt->eval_list_regraft         = NO;
   tree->mod->s_opt->max_delta_lnL_spr_current = 0.0;
@@ -786,22 +775,33 @@ void Global_Spr_Search(t_tree *tree)
   mean_delta_lnL_spr                          = 0.0;
   max_delta_lnL_spr                           = 0.0;
   hit_zero_improv                             = 0;
-  
+  tune_l_mult                                 = 0.01;
+  best_lnL                                    = tree->c_lnL;
   
   iter = 0;
   do
     {
-      if(iter) for(int i=0;i<2*tree->n_otu-3;++i) MIXT_Multiply_Scalar_Dbl(tree->a_edges[i]->l,Rgamma((phydbl)(0.2*iter+1),(phydbl)(1./(0.2*iter+1))));
+      
+      if(!(iter%freq))
+        {
+          for(int i=0;i<2*tree->n_otu-3;++i) MIXT_Multiply_Scalar_Dbl(tree->a_edges[i]->l,Rgamma((phydbl)(tune_l_mult*iter+1),(phydbl)(1./(tune_l_mult*iter+1))));
+          for(int i=0;i<2*tree->n_otu-3;++i) Set_Scalar_Dbl_Min_Thresh(tree->mod->l_min,tree->a_edges[i]->l);
+          for(int i=0;i<2*tree->n_otu-3;++i) Set_Scalar_Dbl_Max_Thresh(tree->mod->l_max,tree->a_edges[i]->l);
+          freq--;
+          if(freq < 1) freq=1;
+        }
+      
+      if(iter> 10) tree->mod->s_opt->l_min_spr = 1.E-3;
+
       Spr(tree->c_lnL,1.0,tree);
       Optimize_Br_Len_Serie(2,tree);
       
-      if(!(iter%15)) Round_Optimize(tree,1000);
-
+      if(!(iter%round_freq) && iter > 0) Round_Optimize(tree,1000);
 
       if(tree->verbose > VL0 && tree->io->quiet == NO)
         {
           time(&t_cur);
-          PhyML_Printf("\n\t%8ds %3d lnL: %12.2f depth: %5d/%5d   # improvements: %5d delta_lnL: %10.2f/%10.2f %2c",
+          PhyML_Printf("\n\t%8ds | %3d | lnL=%12.1f | depth=%5d/%5d | improvements=%4d | delta_lnL=%7.1f/%7.1f | tune=%.2f",
                        (int)(t_cur-tree->t_beg),
                        iter+1,
                        tree->c_lnL,
@@ -810,13 +810,16 @@ void Global_Spr_Search(t_tree *tree)
                        tree->mod->s_opt->n_improvements,
                        tree->mod->s_opt->max_delta_lnL_spr_current,
                        tree->mod->s_opt->max_delta_lnL_spr,
-                       (!(iter%15)) ? '!':' ');
+                       tune_l_mult);
         }
 
-      tree->mod->s_opt->max_depth_path = MAX(5,MAX(tree->mod->s_opt->max_spr_depth+4,(int)(0.8*tree->mod->s_opt->max_depth_path)));
-      tree->mod->s_opt->max_depth_path = MIN(20,tree->mod->s_opt->max_depth_path);
-
+      if(tree->mod->s_opt->n_improvements > (int)(tree->n_otu/5))
+        {
+          tune_l_mult *= 1.2;
+        }
       
+      tree->mod->s_opt->max_depth_path = MAX(5,MAX(tree->mod->s_opt->max_spr_depth+6,(int)(0.6*tree->mod->s_opt->max_depth_path)));
+       
       if((iter%4) > 0 || iter == 0)
         {
           mean_delta_lnL_spr += tree->mod->s_opt->max_delta_lnL_spr_current;
@@ -849,27 +852,107 @@ void Global_Spr_Search(t_tree *tree)
       no_improv++;
       iter++;
     }
-  while(tree->mod->s_opt->n_improvements > 1);
+  while(tree->mod->s_opt->n_improvements > 15 && no_improv < 10);
         
-  
-  tree->mod->s_opt->min_diff_lk_move  = 1.E-1;
-  tree->mod->s_opt->min_diff_lk_local = 1.E-1;
-  Copy_Tree(best_tree,tree);
-  Round_Optimize(tree,1000);
 
-
-
-  
   if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf("\n\n. Second round of optimization...\n");
-  last_best_found = 0;
 
-  tree->mod->s_opt->max_depth_path            = MIN(20,(int)(tree->n_otu/5+1));
-  tree->mod->s_opt->max_delta_lnL_spr         = MAX(100.,tree->mod->s_opt->max_delta_lnL_spr);
+  
+  tree->mod->s_opt->max_depth_path            = MAX(10,(int)(1.5*tree->mod->s_opt->max_depth_path));
+  tree->mod->s_opt->l_min_spr                 = 1.E-3;
   tree->mod->s_opt->spr_lnL                   = YES;
   tree->mod->s_opt->spr_pars                  = NO;
   tree->mod->s_opt->min_diff_lk_move          = 1.E-2;
   tree->mod->s_opt->min_diff_lk_local         = 1.E-2;
-  tree->perform_spr_right_away                = YES;
+  tree->mod->s_opt->apply_spr_right_away      = YES;
+  tree->mod->s_opt->apply_spr                 = YES;
+  tree->mod->s_opt->eval_list_regraft         = NO;
+  tree->mod->s_opt->max_delta_lnL_spr_current = 0.0;
+  tree->mod->s_opt->min_n_triple_moves        = 1;
+  tree->mod->s_opt->deepest_path              = 0;
+  mean_delta_lnL_spr                          = 0.0;
+  max_delta_lnL_spr                           = 0.0;
+  hit_zero_improv                             = 0;
+  no_improv                                   = 0;
+  
+  do
+    {
+      if(!(iter%freq))
+        {
+          for(int i=0;i<2*tree->n_otu-3;++i) MIXT_Multiply_Scalar_Dbl(tree->a_edges[i]->l,Rgamma((phydbl)(tune_l_mult*iter+1),(phydbl)(1./(tune_l_mult*iter+1))));
+          for(int i=0;i<2*tree->n_otu-3;++i) Set_Scalar_Dbl_Min_Thresh(tree->mod->l_min,tree->a_edges[i]->l);
+          for(int i=0;i<2*tree->n_otu-3;++i) Set_Scalar_Dbl_Max_Thresh(tree->mod->l_max,tree->a_edges[i]->l);          
+          freq--;
+          if(freq < 1) freq=1;
+        }
+      
+      Spr(tree->c_lnL,1.0,tree);
+      Optimize_Br_Len_Serie(2,tree);
+      
+      if(!(iter%round_freq) && iter > 0) Round_Optimize(tree,1000);
+
+      if(tree->verbose > VL0 && tree->io->quiet == NO)
+        {
+          time(&t_cur);
+          PhyML_Printf("\n\t%8ds | %3d | lnL=%12.1f | depth=%5d/%5d | improvements=%4d | delta_lnL=%7.1f/%7.1f",
+                       (int)(t_cur-tree->t_beg),
+                       iter+1,
+                       tree->c_lnL,
+                       tree->mod->s_opt->max_spr_depth,
+                       tree->mod->s_opt->max_depth_path,
+                       tree->mod->s_opt->n_improvements,
+                       tree->mod->s_opt->max_delta_lnL_spr_current,
+                       tree->mod->s_opt->max_delta_lnL_spr);
+        }
+
+      tree->mod->s_opt->max_depth_path = MAX(5,MAX(tree->mod->s_opt->max_spr_depth+4,(int)(0.8*tree->mod->s_opt->max_depth_path)));
+      tree->mod->s_opt->max_depth_path = MIN(20,tree->mod->s_opt->max_depth_path);
+      
+      if((iter%4) > 0 || iter == 0)
+        {
+          mean_delta_lnL_spr += tree->mod->s_opt->max_delta_lnL_spr_current;
+          if(tree->mod->s_opt->max_delta_lnL_spr_current > max_delta_lnL_spr) max_delta_lnL_spr = tree->mod->s_opt->max_delta_lnL_spr_current;
+        }
+      else if(iter > 0)
+        {
+          mean_delta_lnL_spr /= 4.0;
+          tree->mod->s_opt->max_delta_lnL_spr = MAX(20.,2.*max_delta_lnL_spr);
+          mean_delta_lnL_spr = tree->mod->s_opt->max_delta_lnL_spr_current;
+          max_delta_lnL_spr = 0.0;
+        }
+      
+      
+      if(tree->c_lnL > best_lnL)
+        {
+          no_improv = 0;
+          best_lnL = tree->c_lnL;
+          Copy_Tree(tree,best_tree);
+          if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf(" +");
+          if(tree->io->print_json_trace == YES) JSON_Tree_Io(tree,tree->io->fp_out_json_trace);
+        }
+      
+      if(tree->mod->s_opt->n_improvements == 0)
+        {
+          hit_zero_improv++;
+        }
+      
+      no_improv++;
+      iter++;
+    }
+  while(tree->mod->s_opt->n_improvements > 5 && no_improv < 10);
+
+  if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf("\n\n. Third round of optimization...\n");
+  last_best_found = 0;
+
+  tree->mod->s_opt->max_depth_path            = MAX(10,tree->mod->s_opt->max_depth_path);
+  tree->mod->s_opt->max_delta_lnL_spr         = MAX(100.,tree->mod->s_opt->max_delta_lnL_spr);
+  tree->mod->s_opt->spr_lnL                   = YES;
+  tree->mod->s_opt->spr_pars                  = NO;
+  tree->mod->s_opt->l_min_spr                 = 1.E-4;
+  tree->mod->s_opt->min_diff_lk_move          = 1.E-2;
+  tree->mod->s_opt->min_diff_lk_local         = 1.E-2;
+  tree->mod->s_opt->apply_spr_right_away      = YES;
+  tree->mod->s_opt->apply_spr                 = YES;
   tree->mod->s_opt->eval_list_regraft         = YES;
   tree->mod->s_opt->max_delta_lnL_spr_current = 0.0;
   tree->mod->s_opt->min_n_triple_moves        = 5;
@@ -881,12 +964,12 @@ void Global_Spr_Search(t_tree *tree)
       Spr(tree->c_lnL,1.0,tree);
       Optimize_Br_Len_Serie(2,tree);
       
-      if(!(iter%15)) Round_Optimize(tree,1000);
+      if(!(iter%round_freq)) Round_Optimize(tree,1000);
 
       if(tree->verbose > VL0 && tree->io->quiet == NO)
         {
           time(&t_cur);
-          PhyML_Printf("\n\t%8ds %3d lnL: %12.2f depth: %5d/%5d   # improvements: %3d delta_lnL: %10.2f/%10.2f   # triple moves: %d %2c",
+          PhyML_Printf("\n\t%8ds | %3d | lnL=%12.1f | depth=%5d/%5d | improvements=%4d | delta_lnL=%7.1f/%7.1f | triple moves=%4d",
                        (int)(t_cur-tree->t_beg),
                        iter+1,
                        tree->c_lnL,
@@ -895,9 +978,7 @@ void Global_Spr_Search(t_tree *tree)
                        tree->mod->s_opt->n_improvements,
                        tree->mod->s_opt->max_delta_lnL_spr_current,
                        tree->mod->s_opt->max_delta_lnL_spr,
-                       tree->mod->s_opt->min_n_triple_moves,
-                       (!(iter%15)) ? '!':' ');
-         
+                       tree->mod->s_opt->min_n_triple_moves);
         }
       
       tree->mod->s_opt->max_depth_path     = MIN(30,MAX(5,MAX(2*tree->mod->s_opt->max_spr_depth,(int)(0.8*tree->mod->s_opt->max_depth_path))));
