@@ -28,6 +28,7 @@ the GNU public licence. See http://www.opensource.org for details.
 #include <float.h>
 #include <assert.h>
 #include <stdbool.h>
+#include <signal.h>
 /* #include <malloc/malloc.h> */
 /* #include <malloc.h> */
 
@@ -88,8 +89,8 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 #endif
 
 
-extern int CALL;
-extern int TIME;
+int CALL;
+int TIME;
 
 #define AC 0
 #define AG 1
@@ -132,10 +133,11 @@ extern int TIME;
 #define  R_MAT_WEIGHT_MAX 100.
 
 #define  E_FRQ_MIN 0.01
-#define  E_FRQ_MAX 1.00
+#define  E_FRQ_MAX 0.99
+
+#define  UNSCALED_E_FRQ_MIN -1000.
+#define  UNSCALED_E_FRQ_MAX +10.
 
-#define  UNSCALED_E_FRQ_MIN 0.01
-#define  UNSCALED_E_FRQ_MAX 100.
 
 #define  TSTV_MIN 0.05
 #define  TSTV_MAX 20.0
@@ -143,8 +145,12 @@ extern int TIME;
 #define  PINV_MIN 0.00001
 #define  PINV_MAX 0.99999
 
-#define RR_MIN 0.0001
-#define RR_MAX 10000.0
+#define RR_MIN 0.01
+#define RR_MAX 100.0
+
+#define UNSCALED_RR_MIN log(RR_MIN)
+#define UNSCALED_RR_MAX log(RR_MAX)
+
 
 #define GAMMA_RR_UNSCALED_MIN 0.01
 #define GAMMA_RR_UNSCALED_MAX 200.
@@ -385,7 +391,7 @@ typedef	double phydbl;
 #define LOGSMALL -690.
 
 
-#if !(defined PHYTIME || defined SERGEII)
+#if !(defined PHYTIME)
 #define BL_MIN 1.E-8
 #define BL_MAX 100.
 #else
@@ -410,6 +416,12 @@ typedef	double phydbl;
 #define LARGE 256
 #define TWO_TO_THE_LARGE 115792089237316195423570985008687907853269984665640564039457584007913129639936.0 /* 2^256 In R : sprintf("%.100f", 2^256)*/
 
+/* #define LARGE 128 */
+/* #define TWO_TO_THE_LARGE 340282366920938463463374607431768211456.0 /\* 2^128 In R : sprintf("%.100f", 2^128)*\/ */
+
+/* #define LARGE 320 */
+/* #define TWO_TO_THE_LARGE 2135987035920910082395021706169552114602704522356652769947041607822219725780640550022962086936576.0 */
+
 #define INV_TWO_TO_THE_LARGE (1./TWO_TO_THE_LARGE)
 
 #define SCALE_RATE_SPECIFIC 1
@@ -516,6 +528,10 @@ typedef struct __Node {
   struct __Node                *next_mixt; /*! Next mixture tree*/
   struct __Node                *prev_mixt; /*! Parent mixture tree */
   struct __Calibration              **cal; /*! List of calibration constraints attached to this node */
+  struct __Lindisk_Node             *ldsk; /*! Used in PhyREX. Lineage/Disk this node corresponds to */
+  struct __Node                  *rk_next; /*! Next node in the list of ranked nodes (from oldest to youngest) */
+  struct __Node                  *rk_prev; /*! Previous node in the list of ranked nodes (from oldest to youngest) */
+
 
   int                           *bip_size; /*! Size of each of the three lists from bip_node */
   int                                 num; /*! t_node number */
@@ -526,7 +542,6 @@ typedef struct __Node {
   int                               n_cal; /*! Number of calibration constraints */
 
   phydbl                           *score; /*! score used in BioNJ to determine the best pair of nodes to agglomerate */
-  phydbl                               *l; /*! lengths of the (three or one) branche(s) connected this t_node */
   phydbl                     dist_to_root; /*! distance to the root t_node */
 
   short int                        common;
@@ -658,6 +673,8 @@ typedef struct __Edge {
 
   phydbl                       ratio_test; /*! approximate likelihood ratio test */
   phydbl                   alrt_statistic; /*! aLRT statistic */
+  phydbl                      support_val;
+
 
   char                           **labels; /*! string of characters that labels the corresponding t_edge */
   int                            n_labels; /*! number of labels */
@@ -698,10 +715,9 @@ typedef struct __Tree{
   struct __Tdraw                     *ps_tree; /*! structure for drawing trees in postscript format */
   struct __T_Rate                       *rates; /*! structure for handling rates of evolution */
   struct __Tmcmc                        *mcmc;
-  struct __Triplet            *triplet_struct;
   struct __Phylogeo                      *geo;
   struct __Migrep_Model                 *mmod;
-  struct __Disk_Event                   *disk;
+  struct __Disk_Event             *young_disk; /*! Youngest disk (i.e., disk which age is the closest to present). Used in PhyREX */
   struct __XML_node                 *xml_root;
   struct __Generic_LL              *edge_list;
   struct __Generic_LL              *node_list;
@@ -712,6 +728,7 @@ typedef struct __Tree{
   short int                         eval_glnL; /*! Evaluate tree likelihood */
   short int                    scaling_method;
   short int                     fully_nni_opt;
+  short int                 numerical_warning;
   
   short int                      use_eigen_lr;
   int                            is_mixt_tree;
@@ -726,10 +743,6 @@ typedef struct __Tree{
   int                                  n_swap; /*! number of NNIs performed */
   int                               n_pattern; /*! number of distinct site patterns */
   int                      has_branch_lengths; /*! =1 iff input tree displays branch lengths */
-  short int                    print_tbe_val;  /*! if print_tbe_val=1,  the TBE bootstrap values are printed */
-  short int                    print_boot_val; /*! if print_boot_val=1, the bootstrap values are printed */
-  short int                    print_alrt_val; /*! if print_alrt_val=1, the aLRT values are printed */
-  short int                    print_node_num; /*! print node numbers if print_node_num=1 */
   short int                        both_sides; /*! both_sides=1 -> a pre-order and a post-order tree
                           traversals are required to compute the likelihood
                           of every subtree in the phylogeny*/
@@ -995,8 +1008,8 @@ typedef struct __Matrix { /*! mostly used in BIONJ */
 
 typedef struct __RateMatrix {
   int                    n_diff_rr; /*! number of different relative substitution rates in the custom model */
-  vect_dbl                     *rr; /*! relative rate parameters of the GTR or custom model (given by rr_val[rr_num[i]]) */
-  vect_dbl                 *rr_val; /*! relative rate parameters of the GTR or custom model */
+  vect_dbl                     *rr; /*! relative rate parameters of the GTR or custom model (rescaled) */
+  vect_dbl                 *rr_val; /*! log of relative rate parameters of the GTR or custom model (unscaled) */
   vect_int                 *rr_num;
   vect_int           *n_rr_per_cat; /*! number of rate parameters in each category */
   vect_dbl                   *qmat;
@@ -1092,7 +1105,6 @@ typedef struct __Model {
   int                    augmented;
   int                           ns; /*! number of states (4 for ADN, 20 for AA) */
 
-  int                    bootstrap; /*! Number of bootstrap replicates (0 : no bootstrap analysis is launched) */
   int                    use_m4mod; /*! Use a Makrkov modulated Markov model ? */
 
   scalar_dbl                *kappa; /*! transition/transversion rate */
@@ -1139,9 +1151,10 @@ typedef struct __Eigen{
   int        *space_int;
   phydbl         *e_val; /*! eigen values (vector), real part. */
   phydbl      *e_val_im; /*! eigen values (vector), imaginary part */
-  phydbl      *r_e_vect; /*! right eigen vector (matrix), real part */
   phydbl   *r_e_vect_im; /*! right eigen vector (matrix), imaginary part */
   phydbl      *l_e_vect; /*! left eigen vector (matrix), real part */
+  phydbl      *r_e_vect; /*! right eigen vector (matrix), real part */
+  phydbl           *dum; /*! dummy ns*ns space */
 
   struct __Eigen  *prev;
   struct __Eigen  *next;
@@ -1208,8 +1221,11 @@ typedef struct __Option { /*! mostly used in 'help.c' */
   char                  *out_ps_file; /*! name of the file in which tree(s) is(are) written */
   FILE                    *fp_out_ps;
 
-  char           *out_ancestral_file; /*! name of the file containing the ancestral sequences */
-  FILE             *fp_out_ancestral; /*! pointer to the file containing the ancestral sequences */
+  char       *out_ancestral_seq_file; /*! name of the file containing the ancestral sequences */
+  char      *out_ancestral_tree_file; /*! name of the file containing the tree with internal node labelled according to refs in ancestral_seq_file */
+
+  FILE         *fp_out_ancestral_seq; /*! pointer to the file containing the ancestral sequences */
+  FILE        *fp_out_ancestral_tree; /*! pointer to the file containing the tree with labels on internal nodes  */
 
   char                *in_coord_file; /*! name of input file containing coordinates */
   FILE                  *fp_in_coord; /*! pointer to the file containing coordinates */
@@ -1278,7 +1294,17 @@ typedef struct __Option { /*! mostly used in 'help.c' */
   int                  has_io_weights;
   int                   tbe_bootstrap;/* Replace standard bootstrap with tbe bootstrap (only when b>0) */
 
-  int leave_duplicates; /* Leave duplicated sequences */
+  int                leave_duplicates;/* Leave duplicated sequences */
+  int                       precision;/* Decimal output precision for values in stats file */
+  int               n_boot_replicates;
+  
+  short int            print_node_num; /*! print node numbers if print_node_num=1 */
+  short int         print_support_val;
+  
+  short int                    do_tbe;
+  short int                   do_boot;
+  short int                   do_alrt;
+
 }option;
 
 /*!********************************************************/
@@ -1358,6 +1384,11 @@ typedef struct __Optimiz { /*! parameters to be optimised (mostly used in 'optim
   int      curr_opt_free_rates;
 
   int          nni_br_len_opt;
+
+  int    apply_spr_right_away;
+  int               apply_spr;
+
+  phydbl            l_min_spr;
 }t_opt;
 
 /*!********************************************************/
@@ -1421,26 +1452,6 @@ typedef struct __SPR{
 
 /*!********************************************************/
 
-typedef struct __Triplet{
-  int    size;
-  phydbl *F_bc;
-  phydbl *F_cd;
-  phydbl *F_bd;
-  phydbl ****core;
-  phydbl ***p_one_site;
-  phydbl ***sum_p_one_site;
-  phydbl *pi_bc;
-  phydbl *pi_cd;
-  phydbl *pi_bd;
-  struct __Eigen *eigen_struct;
-  struct __Model *mod;
-
-  struct __Triplet *next;
-  struct __Triplet *prev;
-}triplet;
-
-/*!********************************************************/
-
 typedef struct __Pnode{
   struct __Pnode **next;
   int weight;
@@ -1480,11 +1491,10 @@ typedef struct __Tdraw {
   int         page_height;
   int      tree_box_width;
 
-  int         *cdf_mat;
+  int            *cdf_mat;
   phydbl       *cdf_mat_x;
   phydbl       *cdf_mat_y;
 
-
   phydbl max_dist_to_root;
 }tdraw;
 
@@ -1678,12 +1688,12 @@ typedef struct __Tmcmc {
   int num_move_phyrex_swap_disk;
   int num_move_phyrex_indel_hit;
   int num_move_phyrex_spr;
+  int num_move_phyrex_spr_local;
   int num_move_phyrex_scale_times;
   int num_move_phyrex_ldscape_lim;
   int num_move_phyrex_sigsq;
   int num_move_phyrex_sim;
   int num_move_phyrex_traj;
-  int num_move_phyrex_lbda_times;
   int num_move_phyrex_indel_disk_serial;
   int num_move_phyrex_sim_plus;
   int num_move_phyrex_indel_hit_serial;
@@ -1692,6 +1702,7 @@ typedef struct __Tmcmc {
   int num_move_phyrex_disk_multi;
   int num_move_phyrex_ldsk_given_disk;
   int num_move_phyrex_disk_given_ldsk;
+  int num_move_phyrex_add_remove_jump;
   int num_move_clade_change;
 
   int nd_t_digits;
@@ -1929,16 +1940,18 @@ typedef struct __Migrep_Model{
 /*!********************************************************/
 
 typedef struct __Disk_Event{
-  struct __Geo_Coord           *centr;
-  phydbl                         time;
-  struct __Disk_Event           *next;
-  struct __Disk_Event           *prev;
-  struct __Lindisk_Node      **ldsk_a; // array of lindisk nodes corresponding to this disk event
-  int                        n_ldsk_a; // size of ldsk_a
-  struct __Lindisk_Node         *ldsk;
-  struct __Migrep_Model         *mmod;
-  char                            *id;
-  phydbl                        c_lnL;
+  struct __Geo_Coord      *centr;
+  phydbl                    time;
+  struct __Disk_Event      *next;
+  struct __Disk_Event      *prev;
+  struct __Lindisk_Node **ldsk_a; // array of lindisk nodes corresponding to this disk event.
+  int                   n_ldsk_a; // size of ldsk_a.
+  struct __Lindisk_Node    *ldsk;
+  struct __Migrep_Model    *mmod;
+  char                       *id;
+
+  phydbl                   c_lnL;
+  short int            age_fixed; // time is fixed for disks corresponding to samples.
 }t_dsk;
 
 /*!********************************************************/
@@ -2062,7 +2075,7 @@ int Assign_State(char *c,int datatype,int stepsize);
 char Reciproc_Assign_State(int i_state,int datatype);
 int Assign_State_With_Ambiguity(char *c,int datatype,int stepsize);
 void Clean_Tree_Connections(t_tree *tree);
-void Bootstrap(t_tree *tree, int tbe_bootstrap);
+void Bootstrap(t_tree *tree);
 void Br_Len_Involving_Invar(t_tree *tree);
 void Br_Len_Not_Involving_Invar(t_tree *tree);
 void Getstring_Stdin(char *s);
@@ -2097,7 +2110,7 @@ void Hide_Ambiguities(calign *data);
 void Copy_Tree(t_tree *ori, t_tree *cpy);
 void Prune_Subtree(t_node *a,t_node *d,t_edge **target,t_edge **residual,t_tree *tree);
 void Graft_Subtree(t_edge *target, t_node *link, t_node *link_daughter, t_edge *residual, t_node *target_nd, t_tree *tree);
-void Reassign_Node_Nums(t_node *a,t_node *d,int *curr_ext_node,int *curr_int_node,t_tree *tree);
+void Reassign_Node_Nums(t_node *a,t_node *d, unsigned int *curr_ext_node, unsigned int *curr_int_node,t_tree *tree);
 void Reassign_Edge_Nums(t_node *a,t_node *d,int *curr_br,t_tree *tree);
 void Find_Mutual_Direction(t_node *n1,t_node *n2,short int *dir_n1_to_n2,short int *dir_n2_to_n1);
 void Update_Dir_To_Tips(t_node *a,t_node *d,t_tree *tree);
@@ -2166,18 +2179,17 @@ t_tree *Dist_And_BioNJ(calign *cdata,t_mod *mod,option *io);
 void Add_BioNJ_Branch_Lengths(t_tree *tree, calign *cdata, t_mod *mod, matrix *mat);
 char *Bootstrap_From_String(char *s_tree,calign *cdata,t_mod *mod,option *io);
 char *aLRT_From_String(char *s_tree,calign *cdata,t_mod *mod,option *io);
-void Prepare_Tree_For_Lk(t_tree *tree);
 void Find_Common_Tips(t_tree *tree1,t_tree *tree2);
 phydbl Get_Tree_Size(t_tree *tree);
-void Dist_To_Root_Pre(t_node *a,t_node *d,t_edge *b,t_tree *tree);
-void Dist_To_Root(t_node *n_root,t_tree *tree);
+void Dist_To_Root_Pre(t_node *a, t_node *d, t_edge *b, t_tree *tree);
+void Dist_To_Root(t_tree *tree);
 char *Basename(char *path);
 t_node *Find_Lca_Pair_Of_Nodes(t_node *n1,t_node *n2,t_tree *tree);
 t_node *Find_Lca_Clade(t_node **node_list,int node_list_size,t_tree *tree);
 int Get_List_Of_Ancestors(t_node *ref_node,t_node **list,int *size,t_tree *tree);
 int Edge_Num_To_Node_Num(int edge_num,t_tree *tree);
-void Time_To_Branch(t_tree *tree);
-void Time_To_Branch_Pre(t_node *a,t_node *d,t_tree *tree);
+void Time_To_Bl(t_tree *tree);
+void Time_To_Bl_Pre(t_node *a, t_node *d, t_edge *b, t_tree *tree);
 void Branch_Lengths_To_Rate_Lengths(t_tree *tree);
 void Branch_Lengths_To_Rate_Lengths_Pre(t_node *a,t_node *d,t_tree *tree);
 int Find_Clade(char **tax_name_list,int list_size,t_tree *tree);
@@ -2230,14 +2242,13 @@ phydbl String_To_Dbl(char *string);
 int String_To_Int(char *string);
 char *To_Upper_String(char *in);
 void Connect_CSeqs_To_Nodes(calign *cdata, option *io, t_tree *tree);
-void Switch_Eigen(int state, t_mod *mod);
 void Joint_Proba_States_Left_Right(phydbl *Pij, phydbl *p_lk_left, phydbl *p_lk_rght,
                    vect_dbl *pi, int scale_left, int scale_rght,
                    phydbl *F, int n, int site, t_tree *tree);
 void Set_Both_Sides(int yesno, t_tree *tree);
 void Set_D_States(calign *data, int datatype, int stepsize);
-void Branch_To_Time(t_tree *tree);
-void Branch_To_Time_Pre(t_node *a, t_node *d, t_tree *tree);
+void Bl_To_Time(t_tree *tree);
+void Bl_To_Time_Pre(t_node *a, t_node *d, t_tree *tree);
 void Path_Length(t_node *dep, t_node *arr, phydbl *len, t_tree *tree);
 phydbl *Dist_Btw_Tips(t_tree *tree);
 void Random_SPRs_On_Rooted_Tree(t_tree *tree);
@@ -2270,7 +2281,9 @@ scalar_dbl *Duplicate_Scalar_Dbl(scalar_dbl *from);
 scalar_dbl *Read_Weights(option *io);
 phydbl Scalar_Elem(int pos, scalar_dbl *scl);
 int Scalar_Len(scalar_dbl *scl);
+void Set_Scalar_Dbl(phydbl val, scalar_dbl *from);
 void Set_Scalar_Dbl_Min_Thresh(phydbl thresh, scalar_dbl *from);
+void Set_Scalar_Dbl_Max_Thresh(phydbl thresh, scalar_dbl *from);
 void List_Of_Regraft_Nodes(t_node *a, t_node *d, phydbl time_thresh, t_ll *list, t_tree *tree);
 void Push_Bottom_Linked_List(void *what, t_ll **list, bool remove_duplicates);
 void Remove_From_Linked_List(t_ll *elem, void *val, t_ll **list);
@@ -2289,11 +2302,35 @@ void Table_Bottom(unsigned int width);
 t_cal *Duplicate_Calib(t_cal *from);
 t_clad *Duplicate_Clade(t_clad *from);
 void Swap_Partial_Lk_Extra(t_edge *b, t_node *d, int whichone, t_tree *tree);
-void Remove_Duplicates(calign *data, t_mod *mod, option *io);
+void Remove_Duplicates(calign *data, option *io, t_tree *tree);
 short int Are_Sequences_Identical(align *seq1, align *seq2);
 char *Mutation_Id(int mut_idx, t_tree *tree);
 void Random_Tax_Idx(t_node *a, t_node *d, int *idx, t_tree *tree);
 void List_Taxa_In_Clade(t_node *a, t_node *d, t_tree *tree);
+void Alias_Subpatt_Pre(t_node *a, t_node *d, t_tree *tree);
+void Alias_Subpatt_Post(t_node *a, t_node *d, t_tree *tree);
+void Alias_One_Subpatt(t_node *a, t_node *d, t_tree *tree);
+void Alias_Subpatt(t_tree *tree);
+void Map_Mutations(t_node *a, t_node *d, int sa, int sd, t_edge *b, int site, int rcat, int *muttype, phydbl *muttime, int *muttax, int *n_mut, t_tree *tree);
+void Set_Update_Eigen(int yesno, t_mod *mod);
+int *Order_Int(const int *u, const int n);
+int *Order_Dbl(const phydbl *u, const int n);
+char Integer_To_IUPAC_Code(int x);
+void Shuffle_Sites(const phydbl prop, align **data, const int n_otu);
+void Multiply_Scalar_Dbl(phydbl mult, scalar_dbl *x);
+void Insert_Duplicates(t_tree *tree);
+void Get_Node_Ranks_From_Dist_To_Root(t_tree *tree);
+void Get_Node_Ranks_From_Times(t_tree *tree);
+void Get_Node_Ranks_From_Tip_Times(t_tree *tree);
+phydbl Tree_Height(t_tree *tree);
+void Post_Inflate_Times_To_Get_Reasonnable_Edge_Lengths(t_node *a, t_node *d, t_edge *b, phydbl min_l, t_tree *tree);
+void  Inflate_Times_To_Get_Reasonnable_Edge_Lengths(phydbl min_l, t_tree *tree);
+void Refactor_Tree(t_tree *tree);
+void Refactor_External(t_node *a, t_node *d, int *idx, t_tree *tree);
+void Refactor_Internal(t_node *a, t_node *d, t_edge *b, int *idx_nd, int *idx_br, t_tree *tree);
+int *Integer_To_Bit(int val, const int ns);
+char *Bit_To_Character_String(int *bit, int ns);
+
 
 #include "xml.h"
 #include "free.h"
@@ -2313,6 +2350,7 @@ void List_Taxa_In_Clade(t_node *a, t_node *d, t_tree *tree);
 #include "nexus.h"
 #include "init.h"
 #include "mcmc.h"
+#include "ancestral.h"
 
 #ifdef GEO
 #include "geo.h"
