@@ -143,14 +143,18 @@ t_tree *XML_Process_Base(char *xml_filename)
 
 # if defined(PHYTIME)
   strcat(io->out_tree_file,"_phytime_tree");
-# else
+# elif defined(PHYREX)
+  strcat(io->out_tree_file,"_phyrex_tree");
+# elif defined(PHYML)
   strcat(io->out_tree_file,"_phyml_tree");
 #endif
 
 
 # if defined(PHYTIME)
   strcat(io->out_stats_file,"_phytime_stats");
-# else
+# elif defined(PHYREX)
+  strcat(io->out_stats_file,"_phyrex_stats");
+# elif defined(PHYML)
   strcat(io->out_stats_file,"_phyml_stats");
 #endif
 
@@ -746,7 +750,7 @@ t_tree *XML_Process_Base(char *xml_filename)
                       ////////////////////////////////////////
                       
                       else if(!strcmp(parent->name,"equfreqs"))
-                        {
+                        {                          
                           /* If n->ds == NULL, the corrresponding node data structure, n->ds, has not */
                           /* been initialized. If not, do nothing. */
                           if(instance->ds->obj == NULL)
@@ -2216,242 +2220,21 @@ char **XML_Read_Clade(xml_node *xnd_clade, t_tree *tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-void DATE_XML(char *xml_filename)
+void XML_Read_Calibration(xml_node *xroot, t_tree *tree)
 {
-  FILE *fp_xml_in;
-  xml_node *xnd,*xnd_dum,*xnd_clade,*xnd_cal,*xroot;
-  t_tree *mixt_tree,*tree;
-  phydbl low,up,*res,alpha_proba_dbl;
-  char *clade_name,*alpha_proba_string,*calib_id,*clade_id,*dum_string;
-  int seed;
-  t_clad *clade;
+  xml_node *xnd_clade,*xnd_cal,*xnd,*xnd_dum;
+  phydbl low,up,alpha_proba_dbl;
   t_cal *cal;
+  t_clad *clade;
+  char *calib_id,*clade_id,*clade_name,*alpha_proba_string;
   int i,j;
-  
+
   clade = NULL;
   cal = NULL;
-  
-  mixt_tree = XML_Process_Base(xml_filename);
-  assert(mixt_tree);
-  
-  mixt_tree->rates = RATES_Make_Rate_Struct(mixt_tree->n_otu);
-  RATES_Init_Rate_Struct(mixt_tree->rates,NULL,mixt_tree->n_otu);
 
-  
-  tree = mixt_tree;
-  do
-    {
-      // All rate stuctures point to the same object
-      tree->rates = mixt_tree->rates;
-      tree = tree->next;
-    }
-  while(tree);
-
-  
-  fp_xml_in = fopen(xml_filename,"r");
-  if(!fp_xml_in)
-    {
-      PhyML_Fprintf(stderr,"\n. Could not find the XML file '%s'.\n",xml_filename);
-      Exit("\n");
-    }
-
-  /* xroot = XML_Load_File(fp_xml_in); */
-  xroot = mixt_tree->xml_root;
-
-  if(xroot == NULL)
-    {
-      PhyML_Fprintf(stderr,"\n. Encountered an issue while loading the XML file.\n");
-      Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
-    }
-
-  xnd = XML_Search_Node_Name("phytime",NO,xroot);
-
-  if(xnd == NULL)
-    {
-      PhyML_Fprintf(stderr,"\n. Cound not find the \"root\" of the XML file (it should have \'phytime\' as tag name).\n");
-      Generic_Exit(__FILE__,__LINE__,__FUNCTION__);
-    }
-
-  dum_string = XML_Get_Attribute_Value(xnd,"mcmc.chain.len");
-  if(dum_string != NULL) mixt_tree->io->mcmc->chain_len = (int)String_To_Dbl(dum_string);
-  
-  dum_string = XML_Get_Attribute_Value(xnd,"mcmc.sample.every");
-  if(dum_string != NULL) mixt_tree->io->mcmc->sample_interval = (int)String_To_Dbl(dum_string);
-
-  dum_string = XML_Get_Attribute_Value(xnd,"mcmc.print.every");
-  if(dum_string != NULL) mixt_tree->io->mcmc->print_every = (int)String_To_Dbl(dum_string);
-
-  dum_string = XML_Get_Attribute_Value(xnd,"mcmc.burnin");
-  if(dum_string != NULL) mixt_tree->io->mcmc->chain_len_burnin = (int)String_To_Dbl(dum_string);
-
-
-  dum_string = XML_Get_Attribute_Value(xnd,"ignore.sequences");
-  if(dum_string != NULL) mixt_tree->eval_alnL = NO;
-
-  dum_string = XML_Get_Attribute_Value(xnd,"ignore.seq");
-  if(dum_string != NULL) mixt_tree->eval_alnL = NO;
-
-  dum_string = XML_Get_Attribute_Value(xnd,"ignore.data");
-  if(dum_string != NULL) mixt_tree->eval_alnL = NO;
-
-
-  dum_string = XML_Get_Attribute_Value(xnd,"mutmap");
-  if(dum_string != NULL)
-    {
-      int select = XML_Validate_Attr_Int(dum_string,6,
-                                         "true","yes","y",
-                                         "false","no","n");
-      if(select < 3) mixt_tree->io->mutmap = YES;
-      else mixt_tree->io->mutmap = NO;
-    }
-
-  
-  
-  
-  // Looking for XML node with rate-across-lineage info
-  xnd = XML_Search_Node_Name("lineagerates",YES,xroot);
-
-  
-  if(xnd == NULL)
-    {
-      PhyML_Fprintf(stdout,"\n. The model of rate variation across lineages is not specified.");
-      PhyML_Fprintf(stdout,"\n. Using the geometric Brownian model (see Guindon, 2012, Syst. Biol.).\n");
-      mixt_tree->rates->model      = GUINDON;
-      mixt_tree->mod->gamma_mgf_bl = YES;
-      strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
-    }
-  else
-    {
-      char *model_name;
-      model_name = XML_Get_Attribute_Value(xnd,"model");
-
-      if(model_name == NULL)
-        {
-          PhyML_Fprintf(stderr,"\n. Please specify a model of rate variation across lineages,");
-          PhyML_Fprintf(stderr,"\n. e.g., <lineagerates model=\"geometricbrownian\"/>.");
-          PhyML_Fprintf(stderr,"\n. See the manual for more options.");
-          assert(FALSE);
-        }
-      else
-        {
-          if(!strcmp(model_name,"geometricbrownian"))
-            {
-              mixt_tree->rates->model      = GUINDON;
-              mixt_tree->mod->gamma_mgf_bl = YES;
-              strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
-            }
-          else if(!strcmp(model_name,"geometric"))
-            {
-              mixt_tree->rates->model      = GUINDON;
-              mixt_tree->mod->gamma_mgf_bl = YES;
-              strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
-            }
-          else if(!strcmp(model_name,"brownian"))
-            {
-              mixt_tree->rates->model      = GUINDON;
-              mixt_tree->mod->gamma_mgf_bl = YES;
-              strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
-            }
-          else if(!strcmp(model_name,"geo"))
-            {
-              mixt_tree->rates->model      = GUINDON;
-              mixt_tree->mod->gamma_mgf_bl = YES;
-              strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
-            }
-          else if(!strcmp(model_name,"lognormal"))
-            {
-              mixt_tree->rates->model      = LOGNORMAL;
-              mixt_tree->mod->gamma_mgf_bl = NO;
-              strcpy(mixt_tree->rates->model_name,"lognormal (uncorrelated)"); 
-            }
-          else if(!strcmp(model_name,"normal"))
-            {
-              mixt_tree->rates->model      = LOGNORMAL;
-              mixt_tree->mod->gamma_mgf_bl = NO;
-              strcpy(mixt_tree->rates->model_name,"lognormal (uncorrelated)"); 
-            }
-          else if(!strcmp(model_name,"strictclock"))
-            {
-              mixt_tree->rates->model      = STRICTCLOCK;
-              mixt_tree->mod->gamma_mgf_bl = NO;
-              strcpy(mixt_tree->rates->model_name,"strict clock"); 
-            }
-          else if(!strcmp(model_name,"clock"))
-            {
-              mixt_tree->rates->model      = STRICTCLOCK;
-              mixt_tree->mod->gamma_mgf_bl = NO;
-              strcpy(mixt_tree->rates->model_name,"strict clock"); 
-            }
-          else
-            {
-              assert(FALSE);
-            }
-        }
-    }
-  
-  // Looking for XML node with rate-across-lineage info
-  xnd = XML_Search_Node_Name("calibration",YES,xroot);
-
-  if(xnd == NULL)
-    {
-      PhyML_Fprintf(stderr,"\n. No calibration information seems to be provided.");
-      PhyML_Fprintf(stderr,"\n. Please amend your XML file. \n");
-      assert(FALSE);
-    }
-  else
-    {
-      assert(xnd->child);
-      if(XML_Search_Node_Name("upper",NO,xnd->child) == NULL && XML_Search_Node_Name("lower",NO,xnd->child) == NULL)
-	{
-	  PhyML_Fprintf(stderr,"\n. There is no calibration information provided. \n");
-	  PhyML_Fprintf(stderr,"\n. Please check your data. \n");
-          assert(FALSE);
-	}
-    }
-
-  
-  // Looking for calibration node(s)
-  xnd = XML_Search_Node_Name("calibration",YES,xroot);
-  
-  if(xnd == NULL)
-    {
-      PhyML_Fprintf(stderr,"\n. No calibration information seems to be provided.");
-      PhyML_Fprintf(stderr,"\n. Please amend your XML file. \n");
-      Exit("\n");
-    }
-  else
-    {
-      if(XML_Search_Node_Name("upper",NO,xnd->child) == NULL && XML_Search_Node_Name("lower",NO,xnd->child) == NULL)
-	{
-	  PhyML_Fprintf(stderr,"\n. There is no calibration information provided. \n");
-	  PhyML_Fprintf(stderr,"\n. Please check your data. \n");
-	  Exit("\n");
-	}
-    }
-
-  
-  MIXT_Check_Model_Validity(mixt_tree);
-  MIXT_Init_Model(mixt_tree);
-  Print_Data_Structure(NO,stdout,mixt_tree);
-  tree = MIXT_Starting_Tree(mixt_tree);
-  Copy_Tree(tree,mixt_tree);
-  Free_Tree(tree);
-  MIXT_Connect_Cseqs_To_Nodes(mixt_tree);
-  MIXT_Init_T_Beg(mixt_tree);
-  MIXT_Chain_Edges(mixt_tree);
-  MIXT_Chain_Nodes(mixt_tree);
-  Prepare_Tree_For_Lk(mixt_tree);
-  Add_Root(mixt_tree->a_edges[0],mixt_tree);  
-  MIXT_Chain_All(mixt_tree);
-  MIXT_Check_Edge_Lens_In_All_Elem(mixt_tree);
-  MIXT_Turn_Branches_OnOff_In_All_Elem(ON,mixt_tree);
-  MIXT_Check_Invar_Struct_In_Each_Partition_Elem(mixt_tree);
-  MIXT_Check_RAS_Struct_In_Each_Partition_Elem(mixt_tree);
-
-  
-  
   xnd = xroot->child;
   assert(xnd);
+
   do
     {
       if(!strcmp(xnd->name,"calibration")) // Found a XML node <calibration>.
@@ -2485,12 +2268,13 @@ void DATE_XML(char *xml_filename)
           cal->upper = -low;
           cal->is_primary = YES;
           
-          mixt_tree->rates->a_cal[mixt_tree->rates->n_cal] = cal;
-          mixt_tree->rates->n_cal++;
+          tree->rates->a_cal[tree->rates->n_cal] = cal;
+          tree->rates->n_cal++;
                           
           xnd_dum = xnd_cal->child;
           do
             {
+              clade_name = NULL;
               if(!strcmp("appliesto",xnd_dum->name)) 
                 {
                   clade_name = XML_Get_Attribute_Value(xnd_dum,"clade.id");
@@ -2519,7 +2303,7 @@ void DATE_XML(char *xml_filename)
                           int clade_size,nd_num;
                           int i;
                           
-                          xclade     = XML_Read_Clade(xnd_clade->child,mixt_tree);
+                          xclade     = XML_Read_Clade(xnd_clade->child,tree);
                           clade_size = XML_Number_Of_Taxa_In_Clade(xnd_clade->child);
 
                           clade = Make_Clade();
@@ -2547,12 +2331,11 @@ void DATE_XML(char *xml_filename)
                           clade->id = (char *)mCalloc(strlen(clade_id)+1,sizeof(char));
                           strcpy(clade->id,clade_id);
                           
-                          nd_num = Find_Clade(clade->tax_list,clade_size,mixt_tree);
-                          clade->target_nd = mixt_tree->a_nodes[nd_num];
+                          nd_num = Find_Clade(clade->tax_list,clade_size,tree);
+                          clade->target_nd = tree->a_nodes[nd_num];
                           
                           clade->tip_list = Make_Target_Tip(clade->n_tax);
-                          Init_Target_Tip(clade,mixt_tree);
-
+                          Init_Target_Tip(clade,tree);
 
                           Free(xclade);
                         }
@@ -2567,15 +2350,22 @@ void DATE_XML(char *xml_filename)
               xnd_dum = xnd_dum->next;
             }
           while(xnd_dum != NULL);
+          if(clade_name == NULL)
+            {
+              PhyML_Fprintf(stderr,"\n. Could not find calibration information for calibration `%s'",cal->id);
+              PhyML_Fprintf(stderr,"\n. Please amend your XML file.");
+              Exit("\n");
+            }
         }
       xnd = xnd->next;
     }
   while(xnd != NULL);
 
+  
   PhyML_Printf("\n\n.......................................................................");
-  for(i=0;i<mixt_tree->rates->n_cal;++i)
+  for(i=0;i<tree->rates->n_cal;++i)
     {
-      cal = mixt_tree->rates->a_cal[i];
+      cal = tree->rates->a_cal[i];
 
       phydbl sum = 0.0;
       for(j=0;j<cal->clade_list_size;j++) sum += cal->alpha_proba_list[j];
@@ -2591,61 +2381,11 @@ void DATE_XML(char *xml_filename)
           PhyML_Printf("\n.......................................................................");
         }
     }
-  
-                          
-  seed = (mixt_tree->io->r_seed < 0)?(time(NULL)):(mixt_tree->io->r_seed);
-  srand(seed);
-  mixt_tree->io->r_seed = seed;
-
-  MIXT_Chain_Cal(mixt_tree);
-
-  res = DATE_MCMC(mixt_tree);
-
-
-  // Cleaning up...
-  RATES_Free_Rates(mixt_tree->rates);
-  RATES_Free_Rates(mixt_tree->extra_tree->rates);
-  MCMC_Free_MCMC(mixt_tree->mcmc);
-  MCMC_Free_MCMC(mixt_tree->extra_tree->mcmc);
-  Free_Mmod(mixt_tree->mmod);
-  Free_Spr_List_One_Edge(mixt_tree);
-  Free_Triplet(mixt_tree->triplet_struct);
-  Free_Tree_Pars(mixt_tree);
-  Free_Tree_Lk(mixt_tree);
-
-  if(mixt_tree->io->fp_out_trees)      fclose(mixt_tree->io->fp_out_trees);
-  if(mixt_tree->io->fp_out_tree)       fclose(mixt_tree->io->fp_out_tree);
-  if(mixt_tree->io->fp_out_stats)      fclose(mixt_tree->io->fp_out_stats);
-  if(mixt_tree->io->fp_out_json_trace) fclose(mixt_tree->io->fp_out_json_trace);
-  Free_Input(mixt_tree->io);
-
-
-  tree = mixt_tree;
-  do
-    {
-      Free_Calign(tree->data);
-      tree = tree->next_mixt;
-    }
-  while(tree);
-
-  tree = mixt_tree;
-  do
-    {
-      Free_Optimiz(tree->mod->s_opt);
-      tree = tree->next;
-    }
-  while(tree);
-
-  
-  Free_Model_Complete(mixt_tree->mod);
-  Free_Model_Basic(mixt_tree->mod);
-  Free_Tree(mixt_tree->extra_tree);  
-  Free_Tree(mixt_tree);  
-  Free(res);
-  XML_Free_XML_Tree(xroot);
-  fclose(fp_xml_in);
 }
 
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
